#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import re

from kubernetes.utils import is_valid_string, filter_model


class KeyToPath(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_keytopath
    """

    def __init__(self, model=None):
        super(KeyToPath, self).__init__()

        self._key = None
        self._path = None
        self._mode = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'key' in model:
            self.key = model['key']
        if 'path' in model:
            self.path = model['path']
        if 'mode' in model:
            self.mode = model['mode']

    # ------------------------------------------------------------------------------------- key

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, key=None):
        if not is_valid_string(key):
            raise SyntaxError('KeyToPath: key: [ {0} ] is invalid.'.format(key))
        self._key = key

    # ------------------------------------------------------------------------------------- path

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path=None):
        if not is_valid_string(path):
            raise SyntaxError('KeyToPath: path: [ {0} ] is invalid.'.format(path))
        if re.match('/', path):
            raise SyntaxError('KeyToPath: path: [ {0} ] is invalid. It may not be an absolute path'.format(path))
        if re.search('\.\.', path):
            raise SyntaxError('KeyToPath: path: [ {0} ] is invalid. It may not contain the string ".."'.format(path))
        self._path = path

    # ------------------------------------------------------------------------------------- mode

    @property
    def mode(self):
        return self._mode

    @mode.setter
    def mode(self, mode=None):
        if is_valid_string(mode):
            try:
                mode = int(mode)
            except ValueError:
                raise SyntaxError('KeyToPath: mode: [ {0} ] is invalid.'.format(mode))
        if not isinstance(mode, int):
            raise SyntaxError('KeyToPath: mode: [ {0} ] is invalid.'.format(mode))
        self._mode = mode

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.key is not None:
            data['key'] = self.key
        if self.path is not None:
            data['path'] = self.path
        if self.mode is not None:
            data['mode'] = self.mode
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.models.v1.PodSpec import PodSpec


class PodTemplateSpec(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_podtemplatespec
    """

    def __init__(self, model=None):
        super(PodTemplateSpec, self).__init__()

        self._metadata = ObjectMeta()
        self._spec = PodSpec()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'metadata' in model:
            self.metadata = ObjectMeta(model['metadata'])
        if 'spec' in model:
            self.spec = PodSpec(model['spec'])

    # ------------------------------------------------------------------------------------- metadata

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, metadata=None):
        if not isinstance(metadata, ObjectMeta):
            raise SyntaxError('PodTemplateSpec: metadata: [ {0} ] is invalid'.format(metadata))
        self._metadata = metadata

    # ------------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, spec=None):
        if not isinstance(spec, PodSpec):
            raise SyntaxError('PodTemplateSpec: spec: [ {0} ] is invalid'.format(spec))
        self._spec = spec

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.metadata is not None:
            data['metadata'] = self.metadata.serialize()
        if self.spec is not None:
            data['spec'] = self.spec.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1beta1.RollbackConfig import RollbackConfig
from kubernetes.utils import is_valid_string, is_valid_dict


class DeploymentRollback(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_deploymentrollback
    """

    def __init__(self, model=None):
        super(DeploymentRollback, self).__init__()

        self._kind = 'DeploymentRollback'
        self._api_version = 'extensions/v1beta1'

        self._name = None
        self._updated_annotations = {}
        self._rollback_to = RollbackConfig()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'kind' in model:
            self.kind = model['kind']
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'name' in model:
            self.name = model['name']
        if 'updatedAnnotations' in model:
            self.updated_annotations = model['updatedAnnotations']
        if 'rollbackTo' in model:
            self.rollback_to = model['rollbackTo']

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, kind=None):
        if not is_valid_string(kind):
            raise SyntaxError('DeploymentRollback: kind: [ {} ] is invalid.'.format(kind))
        self._kind = kind

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('DeploymentRollback: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('DeploymentRollback: name: [ {} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- updatedAnnotations

    @property
    def updated_annotations(self):
        return self._updated_annotations

    @updated_annotations.setter
    def updated_annotations(self, anns=None):
        if not is_valid_dict(anns):
            raise SyntaxError('DeploymentRollback: updated_annotations: [ {} ] is invalid.'.format(anns))
        self._updated_annotations = anns

    # ------------------------------------------------------------------------------------- rollbackTo

    @property
    def rollback_to(self):
        return self._rollback_to

    @rollback_to.setter
    def rollback_to(self, rt=None):
        if not isinstance(rt, RollbackConfig):
            raise SyntaxError('DeploymentRollback: rollback_to: [ {} ] is invalid.'.format(rt))
        self._rollback_to = rt

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.kind is not None:
            data['kind'] = self.kind
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.name is not None:
            data['name'] = self.name
        if self.updated_annotations is not None:
            data['updatedAnnotations'] = self.updated_annotations
        if self.rollback_to is not None:
            data['rollbackTo'] = self.rollback_to.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json
import time

from kubernetes.K8sExceptions import DrainNodeException, TimedOutException, NotFoundException
from kubernetes.K8sNamespace import K8sNamespace
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPod import K8sPod
from kubernetes.K8sConfig import K8sConfig
from kubernetes.models.v1.Node import Node
from kubernetes.models.v1.NodeCondition import NodeCondition
from kubernetes.utils import is_valid_string, is_valid_list
from kubernetes.models.v1.Taint import Taint


class K8sNode(K8sObject):

    DRAIN_WAIT_TIMEOUT_SECONDS = 60

    def __init__(self, config=None, name=None):
        super(K8sNode, self).__init__(
            config=config,
            name=name,
            obj_type='Node'
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sNode, self).create()
        self.get()
        return self

    def update(self):
        super(K8sNode, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sNode, self).list(labels=labels)
        nodes = list(map(lambda x: Node(x), ls))
        if pattern is not None:
            nodes = list(filter(lambda x: pattern in x.name, nodes))
        k8s = []
        for x in nodes:
            j = K8sNode(config=self.config, name=x.name).from_model(x)
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Node(self.get_model())
        return self

    def get_annotation(self, k=None):
        if k in self.model.metadata.annotations:
            return self.model.metadata.annotations[k]
        return None

    def get_label(self, k=None):
        if k in self.model.metadata.labels:
            return self.model.metadata.labels[k]
        return None

    # ------------------------------------------------------------------------------------- pod_cidr

    @property
    def pod_cidr(self):
        return self.model.spec.pod_cidr

    @pod_cidr.setter
    def pod_cidr(self, v=None):
        self.model.spec.pod_cidr = v

    # ------------------------------------------------------------------------------------- external_id

    @property
    def external_id(self):
        return self.model.spec.external_id

    @external_id.setter
    def external_id(self, v=None):
        self.model.spec.external_id = v

    # ------------------------------------------------------------------------------------- provider_id

    @property
    def provider_id(self):
        return self.model.spec.provider_id

    @provider_id.setter
    def provider_id(self, v=None):
        self.model.spec.provider_id = v

    # ------------------------------------------------------------------------------------- unschedulable

    @property
    def unschedulable(self):
        return self.model.spec.unschedulable

    @unschedulable.setter
    def unschedulable(self, v=None):
        self.model.spec.unschedulable = v

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self.model.status

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name

    # ------------------------------------------------------------------------------------- is_ready

    @property
    def is_ready(self):
        rc = False
        if self.model.status.conditions is not None:
            for c in self.model.status.conditions:
                assert isinstance(c, NodeCondition)
                if c.condition_type == 'Ready' and c.status == 'True':
                    rc = True
                    break
                else:
                    continue
        return rc

    # ------------------------------------------------------------------------------------- filter

    @staticmethod
    def get_by_name(config=None, name=None):
        nodes = K8sNode(config=config, name=name).list()
        filtered = list(filter(lambda x: x.name == name, nodes))
        if filtered:
            return filtered[0]
        return None

    @staticmethod
    def get_by_labels(config=None, labels=None):

        if config is not None and not isinstance(config, K8sConfig):
            raise SyntaxError(
                'K8sNode.get_by_labels(): config: [ {0} ] is invalid.'.format(config))
        if not isinstance(labels, dict):
            raise SyntaxError(
                'K8sNode.get_by_labels() labels: [ {0} ] is invalid.'.format(labels))

        node_list = list()
        nodes = K8sNode(config=config, name='whatever').list(labels=labels)

        for n in nodes:
            try:
                model = Node(n)
                obj = K8sNode(config=config, name=model.metadata.name).from_model(n)
                node_list.append(obj.get())
            except NotFoundException:
                pass

        return node_list

    # ------------------------------------------------------------------------------------- pods

    @property
    def pods(self):
        return self._pod_inventory()

    @pods.setter
    def pods(self, p=None):
        raise NotImplementedError("K8sNode: pods is read-only.")

    # ------------------------------------------------------------------------------------- drain

    def drain(self, ignore_daemonsets=False, delete_local_storage=False, force=False):
        """
        Removes all K8sPods from this K8sNode, 
        and prevents additional K8sPods from being scheduled.
        
        :param ignore_daemonsets: a boolean. 
                If false, will fail if a K8sDaemonSet-managed K8sPod is present.
                If true, will continue even if a K8sDaemonSet-managed K8sPod is present.
                        
        :param delete_local_storage: a boolean.
                If false, will fail if a K8sVolume of type 'emptyDir' is found.
                If true, will continue even if an 'emptyDir' K8sVolume is found.
        
        :param force: a boolean.
                If false, will fail if any K8sPods unmanaged by a parent object are found.
                If true, will continue and any unmanaged K8sPods are lost.
        
        :return: self. 
        """

        # inventory of K8sPods found on this node.
        daemonset_pods = []
        pods = self._pod_inventory()

        # cordon the node.
        self.unschedulable = True
        self.update()

        # loop through all pods and delete them.
        for pod in pods:
            if self._is_daemonset(pod):
                if not ignore_daemonsets:
                    raise DrainNodeException("K8sNode: pod: [ {} ] is managed by a DaemonSet.".format(pod.name))
                else:
                    daemonset_pods.append(pod)
                    continue
            if self._has_local_storage(pod) and not delete_local_storage:
                raise DrainNodeException("K8sNode: pod: [ {} ] has local storage that will be lost.".format(pod.name))
            if self._is_orphan(pod) and not force:
                raise DrainNodeException("K8sNode: pod: [ {} ] is unmanaged and will be lost.".format(pod.name))

            pod.delete()

        self._wait_for_pod_deletion(daemonset_pods)

        return self

    def _pod_inventory(self):
        """
        Returns the list of all K8sPods found on this K8sNode.
        
        :return: A list of K8sPods.
        """

        pods = []
        namespaces = K8sNamespace(config=self.config, name="yo").list()
        for ns in namespaces:
            cfg = self.config
            cfg.namespace = ns.name
            p = K8sPod(config=cfg, name="yo").list()
            filtered = filter(lambda x: x.node_name == self.name, p)
            pods += filtered
        return pods

    def _is_daemonset(self, pod=None):
        """
        Determines if a K8sPod is part of a K8sDaemonSet.
        
        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        if 'kubernetes.io/created-by' in pod.annotations:
            parent = json.loads(pod.annotations['kubernetes.io/created-by'])
            if parent['reference']['kind'] == 'DaemonSet':
                return True
        return False

    def _has_local_storage(self, pod=None):
        """
        Determines if a K8sPod has any local storage susceptible to be lost.

        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        for vol in pod.volumes:
            if vol.emptyDir is not None:
                return True
        return False

    def _is_orphan(self, pod=None):
        """
        Determines if a K8sPod is unmanaged by a parent object, and is susceptible to be lost.

        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        if 'kubernetes.io/created-by' not in pod.annotations:
            return True
        return False

    def _wait_for_pod_deletion(self, daemonset_pods=None):
        """
        Wait until this K8sNode has evicted all its K8sPods.
        
        :param daemonset_pods: A list of K8sPods on this K8sNode that are managed by a K8sDaemonSet.
        
        :return: None
        """

        pods = self._pod_inventory()
        start_time = time.time()
        while len(pods) > 0:
            if len(pods) == len(daemonset_pods):
                break
            pods = self._pod_inventory()
            self._check_timeout(start_time)
            time.sleep(1)
        return

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.DRAIN_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException("Timed out draining K8sNode: [ {0} ]".format(self.name))

    # ------------------------------------------------------------------------------------- uncordon

    def uncordon(self):
        """
        Returns this K8sNode into the pool addressable by the kube-scheduler.
        
        :return: self
        """

        self.unschedulable = False
        self.update()
        return self

    # ------------------------------------------------------------------------------------- taint

    @property
    def taints(self):
        return self.model.spec.taints

    @taints.setter
    def taints(self, t=None):
        if not is_valid_list(t, Taint):
            raise SyntaxError('K8sNode: taints: [ {} ] is invalid.'.format(t))
        self.model.spec.taints = t

    def taint(self, key=None, value=None, effect=None):
        if not (key and value and effect):
            raise SyntaxError('K8sNode: taint: you must specify a key, a value and an effect.')
        if not is_valid_string(key) or not is_valid_string(value):
            raise SyntaxError('K8sNode: taint: key: [ {} ] or value: [ {} ] is invalid.'.format(key, value))
        if effect not in Taint.VALID_TAINT_EFFECTS:
            raise SyntaxError('K8sNode: taint: effect must be in {}'.format(Taint.VALID_TAINT_EFFECTS))

        t = Taint()
        t.key = key
        t.value = value
        t.effect = effect

        exists = False
        for existing_taint in self.taints:
            if existing_taint.key == key and existing_taint.value == value and existing_taint.effect == effect:
                exists = True
        if not exists:
            self.taints.append(t)
            self.update()
        return self

    def untaint(self, key=None, value=None):
        if key and value:
            if not is_valid_string(key) or not is_valid_string(value):
                raise SyntaxError('K8sNode: taint: key: [ {} ] or value: [ {} ] is invalid.'.format(key, value))

        remaining_taints = []
        for t in self.taints:
            if key and value:
                if t.key != key and t.value != value:
                    remaining_taints.append(t)

        self.taints = remaining_taints
        self.update()
        return self



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ServicePort import ServicePort
from kubernetes.utils import *


class ServiceSpec(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_servicespec
    """

    VALID_TYPES = ['ExternalName', 'ClusterIP', 'NodePort', 'LoadBalancer']
    VALID_SESSION_AFFINITIES = ['ClientIP', 'None']

    def __init__(self, model=None):
        super(ServiceSpec, self).__init__()

        self._cluster_ip = None
        self._external_ips = []
        self._external_name = None
        self._load_balancer_ip = None
        self._load_balancer_source_ranges = None
        self._ports = []
        self._selector = None
        self._session_affinity = 'None'
        self._type = 'ClusterIP'

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'clusterIP' in model:
            self.cluster_ip = model['clusterIP']
        if 'externalIPs' in model:
            self.external_ips = model['externalIPs']
        if 'externalName' in model:
            self.external_name = model['externalName']
        if 'loadBalancerIP' in model:
            self.load_balancer_ip = model['loadBalancerIP']
        if 'loadBalancerSourceRanges' in model:
            self.load_balancer_source_ranges = model['loadBalancerSourceRanges']
        if 'ports' in model:
            ports = []
            for p in model['ports']:
                port = ServicePort(model=p)
                ports.append(port)
            self.ports = ports
        if 'selector' in model:
            self.selector = model['selector']
        if 'sessionAffinity' in model:
            self.session_affinity = model['sessionAffinity']
        if 'type' in model:
            self.type = model['type']

    # ------------------------------------------------------------------------------------- clusterIP

    @property
    def cluster_ip(self):
        return self._cluster_ip

    @cluster_ip.setter
    def cluster_ip(self, ip=None):
        if not is_valid_string(ip):
            raise SyntaxError('ServiceSpec: cluster_ip: [ {0} ] is invalid.'.format(ip))
        self._cluster_ip = ip

    # ------------------------------------------------------------------------------------- externalIPs

    @property
    def external_ips(self):
        return self._external_ips

    @external_ips.setter
    def external_ips(self, ips=None):
        msg = 'ServiceSpec: external_ips: [ {0} ] is invalid.'.format(ips)
        if not is_valid_list(ips, str):
            raise SyntaxError(msg)
        for ip in ips:
            if not is_valid_ip(ip):
                raise SyntaxError(msg)
        self._external_ips = ips

    # ------------------------------------------------------------------------------------- externalName

    @property
    def external_name(self):
        return self._external_name

    @external_name.setter
    def external_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ServiceSpec: external_name: [ {0} ] is invalid.'.format(name))
        self._external_name = name

    # ------------------------------------------------------------------------------------- loadBalancerIP

    @property
    def load_balancer_ip(self):
        return self._load_balancer_ip

    @load_balancer_ip.setter
    def load_balancer_ip(self, ip=None):
        if not is_valid_string(ip) or not is_valid_ip(ip):
            raise SyntaxError('ServiceSpec: load_balancer_ip: [ {0} ] is invalid.'.format(ip))
        self._load_balancer_ip = ip

    # ------------------------------------------------------------------------------------- loadBalancerSourceRanges

    @property
    def load_balancer_source_ranges(self):
        return self._load_balancer_source_ranges

    @load_balancer_source_ranges.setter
    def load_balancer_source_ranges(self, ranges=None):
        if not is_valid_list(ranges, str):
            raise SyntaxError('ServiceSpec: load_balancer_source_ranges: [ {0} ] is invalid.'.format(ranges))
        self._load_balancer_source_ranges = ranges

    # ------------------------------------------------------------------------------------- ports

    @property
    def ports(self):
        return self._ports

    @ports.setter
    def ports(self, ports=None):
        if not is_valid_list(ports, ServicePort):
            raise SyntaxError('ServiceSpec: ports: [ {0} ] is invalid.'.format(ports))
        self._ports = ports

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, sel=None):
        if not is_valid_dict(sel):
            raise SyntaxError('ServiceSpec: selector: [ {0} ] is invalid.'.format(sel))
        self._selector = sel

    # ------------------------------------------------------------------------------------- session affinity

    @property
    def session_affinity(self):
        return self._session_affinity

    @session_affinity.setter
    def session_affinity(self, affinity=None):
        if not is_valid_string(affinity) or affinity not in ServiceSpec.VALID_SESSION_AFFINITIES:
            raise SyntaxError('ServiceSpec: affinity: [ {0} ] is invalid.'.format(affinity))
        self._session_affinity = affinity

    # ------------------------------------------------------------------------------------- session affinity

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, type=None):
        if not is_valid_string(type) or type not in ServiceSpec.VALID_TYPES:
            raise SyntaxError('ServiceSpec: type: [ {0} ] is invalid.'.format(type))
        self._type = type

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.ports:
            data['ports'] = [x.serialize() for x in self.ports]
        if self.session_affinity:
            data['sessionAffinity'] = self.session_affinity
        if self.type:
            data['type'] = self.type
        if self.cluster_ip is not None:
            data['clusterIP'] = self.cluster_ip
        if self.external_ips is not None:
            data['externalIPs'] = self.external_ips
        if self.external_name is not None:
            data['externalName'] = self.external_name
        if self.load_balancer_ip is not None:
            data['loadBalancerIP'] = self.load_balancer_ip
        if self.load_balancer_source_ranges is not None:
            data['loadBalancerSourceRanges'] = self.load_balancer_source_ranges
        if self.selector is not None:
            data['selector'] = self.selector
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class JobCondition(object):
    """
    http://kubernetes.io/docs/api-reference/batch/v1/definitions/#_v1_jobcondition
    """

    def __init__(self, model=None):
        super(JobCondition, self).__init__()

        self._type = None
        self._status = None
        self._last_probe_time = None
        self._last_transition_time = None
        self._reason = None
        self._message = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'type' in model:
            self.type = model['type']
        if 'status' in model:
            self.status = model['status']
        if 'lastProbeTime' in model:
            self.last_probe_time = model['lastProbeTime']
        if 'lastTransitionTime' in model:
            self.last_transition_time = model['lastTransitionTime']
        if 'reason' in model:
            self.reason = model['reason']
        if 'message' in model:
            self.message = model['message']

    # --------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('JobCondition: type: [ {} ] is invalid.'.format(t))
        self._type = t

    # --------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, s=None):
        if not is_valid_string(s):
            raise SyntaxError('JobCondition: status: [ {} ] is invalid.'.format(s))
        self._status = s

    # --------------------------------------------------------------------------------- lastProbeTime

    @property
    def last_probe_time(self):
        return self._last_probe_time

    @last_probe_time.setter
    def last_probe_time(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('JobCondition: last_probe_time: [ {} ] is invalid.'.format(t))
        self._last_probe_time = t

    # --------------------------------------------------------------------------------- lastTransitionTime

    @property
    def last_transition_time(self):
        return self._last_transition_time

    @last_transition_time.setter
    def last_transition_time(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('JobCondition: last_transition_time: [ {} ] is invalid.'.format(t))
        self._last_transition_time = t

    # --------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, r=None):
        if not is_valid_string(r):
            raise SyntaxError('JobCondition: reason: [ {} ] is invalid.'.format(r))
        self._reason = r

    # --------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, m=None):
        if not is_valid_string(m):
            raise SyntaxError('JobCondition: message: [ {} ] is invalid.'.format(m))
        self._message = m

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.type is not None:
            data['type'] = self.type
        if self.status is not None:
            data['status'] = self.status
        if self.last_probe_time is not None:
            data['lastProbeTime'] = self.last_probe_time
        if self.last_transition_time is not None:
            data['lastTransitionTime'] = self.last_transition_time
        if self.reason is not None:
            data['reason'] = self.reason
        if self.message is not None:
            data['message'] = self.message
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ObjectFieldSelector import ObjectFieldSelector
from kubernetes.models.v1.ResourceFieldSelector import ResourceFieldSelector
from kubernetes.models.v1.ConfigMapKeySelector import ConfigMapKeySelector
from kubernetes.models.v1.SecretKeySelector import SecretKeySelector


class EnvVarSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_envvarsource
    """

    def __init__(self, model=None):
        super(EnvVarSource, self).__init__()

        self._field_ref = None
        self._resource_field_ref = None
        self._config_map_key_ref = None
        self._secret_key_ref = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'fieldRef' in model:
            self.field_ref = ObjectFieldSelector(model['fieldRef'])
        if 'resourceFieldRef' in model:
            self.resource_field_ref = ResourceFieldSelector(model['resourceFieldRef'])
        if 'configMapKeyRef' in model:
            self.config_map_key_ref = ConfigMapKeySelector(model['configMapKeyRef'])
        if 'secretKeyRef' is model:
            self.secret_key_ref = SecretKeySelector(model['secretKeyRef'])

    # ------------------------------------------------------------------------------------- fieldRef

    @property
    def field_ref(self):
        return self._field_ref

    @field_ref.setter
    def field_ref(self, ref=None):
        if not isinstance(ref, ObjectFieldSelector):
            raise SyntaxError('EnvVarSource: field_ref: [ {} ] is invalid.'.format(ref))
        self._field_ref = ref

    # ------------------------------------------------------------------------------------- resourceFieldRef

    @property
    def resource_field_ref(self):
        return self._resource_field_ref

    @resource_field_ref.setter
    def resource_field_ref(self, ref=None):
        if not isinstance(ref, ResourceFieldSelector):
            raise SyntaxError('EnvVarSource: resource_field_ref: [ {} ] is invalid.'.format(ref))
        self._resource_field_ref = ref

    # ------------------------------------------------------------------------------------- configMapKeyRef

    @property
    def config_map_key_ref(self):
        return self._config_map_key_ref

    @config_map_key_ref.setter
    def config_map_key_ref(self, ref=None):
        if not isinstance(ref, ConfigMapKeySelector):
            raise SyntaxError('EnvVarSource: config_map_key_ref: [ {} ] is invalid.'.format(ref))
        self._config_map_key_ref = ref

    # ------------------------------------------------------------------------------------- secretKeyRef

    @property
    def secret_key_ref(self):
        return self._secret_key_ref

    @secret_key_ref.setter
    def secret_key_ref(self, ref=None):
        if not isinstance(ref, SecretKeySelector):
            raise SyntaxError('EnvVarSource: secret_key_ref: [ {} ] is invalid.'.format(ref))
        self._secret_key_ref = ref

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.field_ref is not None:
            data['fieldRef'] = self.field_ref.serialize()
        if self.resource_field_ref is not None:
            data['resourceFieldRef'] = self.resource_field_ref.serialize()
        if self.config_map_key_ref  is not None:
            data['configMapKeyRef'] = self.config_map_key_ref.serialize()
        if self.secret_key_ref is not None:
            data['secretKeyRef'] = self.secret_key_ref.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.models.v1.AttachedVolume import AttachedVolume
from kubernetes.models.v1.ContainerImage import ContainerImage
from kubernetes.models.v1.NodeAddress import NodeAddress
from kubernetes.models.v1.NodeCondition import NodeCondition
from kubernetes.models.v1.NodeDaemonEndpoints import NodeDaemonEndpoints
from kubernetes.models.v1.NodeSystemInfo import NodeSystemInfo
from kubernetes.utils import filter_model


class NodeStatus(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodestatus
    """

    def __init__(self, model=None):
        super(NodeStatus, self).__init__()

        self._capacity = None
        self._allocatable = None
        self._phase = None
        self._conditions = None
        self._addresses = None
        self._daemon_endpoints = None
        self._node_info = None
        self._images = None
        self._volumes_in_use = None
        self._volumes_attached = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'capacity' in model:
            self.capacity = model['capacity']
        if 'allocatable' in model:
            self.allocatable = model['allocatable']
        if 'phase' in model:
            self.phase = model['phase']
        if 'conditions' in model:
            if isinstance(model['conditions'], list):
                conditions = list()
                for c in model['conditions']:
                    conditions.append(NodeCondition(c))
                self.conditions = conditions
            else:
                raise SyntaxError('NodeStatus: conditions model: [ {0} ] is invalid.'.format(model['conditions']))
        if 'addresses' in model:
            if isinstance(model['addresses'], list):
                addresses = list()
                for a in model['addresses']:
                    addresses.append(NodeAddress(a))
                self.addresses = addresses
            else:
                raise SyntaxError('NodeStatus: addresses model: [ {0} ] is invalid.'.format(model['addresses']))
        if 'daemonEndpoints' in model:
            if isinstance(model['daemonEndpoints'], dict):
                self.daemon_endpoints = NodeDaemonEndpoints(model['daemonEndpoints'])
        if 'nodeInfo' in model:
            if isinstance(model['nodeInfo'], dict):
                self.node_info = NodeSystemInfo(model['nodeInfo'])
        if 'images' in model:
            if isinstance(model['images'], list):
                images = list()
                for i in model['images']:
                    images.append(ContainerImage(i))
                self.images = images
            else:
                raise SyntaxError('NodeStatus: images model: [ {0} ] is invalid.'.format(model['images']))
        if 'volumesInUse' in model:
            if isinstance(model['volumesInUse'], list):
                self.volumes_in_use = model['volumesInUse']
        if 'volumesAttached' in model:
            if isinstance(model['volumesAttached'], list):
                volumes = list()
                for v in model['volumesAttached']:
                    volumes.append(AttachedVolume(v))
                self.volumes_attached = volumes
            else:
                raise SyntaxError('NodeStatus: volumesAttached model: [ {0} ] is invalid.'
                                  .format(model['volumesAttached']))
            self.volumes_attached = model['volumesAttached']

    # ------------------------------------------------------------------------------------- capacity

    @property
    def capacity(self):
        return self._capacity

    @capacity.setter
    def capacity(self, v):
        if not isinstance(v, dict):
            raise SyntaxError('NodeStatus: capacity: [ {0} ] is invalid.'.format(v))
        self._capacity = v

    # ------------------------------------------------------------------------------------- allocatable

    @property
    def allocatable(self):
        return self._allocatable

    @allocatable.setter
    def allocatable(self, v):
        if not isinstance(v, dict):
            raise SyntaxError('NodeStatus: allocatable: [ {0} ] is invalid.'.format(v))
        self._allocatable = v

    # ------------------------------------------------------------------------------------- phase

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeStatus: phase: [ {0} ] is invalid.'.format(v))
        self._phase = v

    # ------------------------------------------------------------------------------------- conditions

    @property
    def conditions(self):
        return self._conditions

    @conditions.setter
    def conditions(self, v):
        if not isinstance(v, list):
            raise SyntaxError('NodeStatus: conditions: [ {0} ] is invalid.'.format(v))
        self._conditions = v

    # ------------------------------------------------------------------------------------- addresses

    @property
    def addresses(self):
        return self._addresses

    @addresses.setter
    def addresses(self, v):
        if not isinstance(v, list):
            raise SyntaxError('NodeStatus: addresses: [ {0} ] is invalid.'.format(v))
        self._addresses = v

    # ------------------------------------------------------------------------------------- daemon endpoints

    @property
    def daemon_endpoints(self):
        return self._daemon_endpoints

    @daemon_endpoints.setter
    def daemon_endpoints(self, v):
        if not isinstance(v, NodeDaemonEndpoints):
            raise SyntaxError('NodeStatus: daemon_endpoints: [ {0} ] is invalid.'.format(v))
        self._daemon_endpoints = v

    # ------------------------------------------------------------------------------------- node info

    @property
    def node_info(self):
        return self._node_info

    @node_info.setter
    def node_info(self, v):
        if not isinstance(v, NodeSystemInfo):
            raise SyntaxError('NodeStatus: node_info: [ {0} ] is invalid.'.format(v))
        self._node_info = v

    # ------------------------------------------------------------------------------------- images

    @property
    def images(self):
        return self._images

    @images.setter
    def images(self, v):
        if not isinstance(v, list):
            raise SyntaxError('NodeStatus: images: [ {0} ] is invalid.'.format(v))
        self._images = v

    # ------------------------------------------------------------------------------------- volumes in use

    @property
    def volumes_in_use(self):
        return self._volumes_in_use

    @volumes_in_use.setter
    def volumes_in_use(self, v):
        if not isinstance(v, list):
            raise SyntaxError('NodeStatus: volumes_in_use: [ {0} ] is invalid.'.format(v))
        self._volumes_in_use = v

    # ------------------------------------------------------------------------------------- volumes attached

    @property
    def volumes_attached(self):
        return self._volumes_attached

    @volumes_attached.setter
    def volumes_attached(self, v):
        if not isinstance(v, list):
            raise SyntaxError('NodeStatus: volumes_attached: [ {0} ] is invalid.'.format(v))
        self._volumes_attached = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.capacity:
            data['capacity'] = self.capacity
        if self.allocatable:
            data['allocatable'] = self.allocatable
        if self.phase:
            data['phase'] = self.phase
        if self.conditions:
            l = list()
            for c in self.conditions:
                assert isinstance(c, NodeCondition)
                l.append(c.serialize())
            data['conditions'] = l
        if self.addresses:
            l = list()
            for a in self.addresses:
                assert isinstance(a, NodeAddress)
                l.append(a.serialize())
            data['addresses'] = l
        if self.daemon_endpoints:
            data['daemonEndpoints'] = self.daemon_endpoints.serialize()
        if self.node_info:
            data['nodeInfo'] = self.node_info.serialize()
        if self.images:
            l = list()
            for i in self.images:
                assert isinstance(i, ContainerImage)
                l.append(i.serialize())
            data['images'] = l
        if self.volumes_in_use:
            data['volumesInUse'] = self.volumes_in_use
        if self.volumes_attached:
            l = list()
            for v in self.volumes_attached:
                assert isinstance(v, AttachedVolume)
                l.append(v.serialize())
            data['volumesAttached'] = l
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class ReplicaSetStatus(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_replicasetstatus
    """

    def __init__(self, model=None):
        super(ReplicaSetStatus, self).__init__()

        self._replicas = 0
        self._fully_labeled_replicas = None
        self._ready_replicas = None
        self._observed_generation = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'replicas' in model:
            self.replicas = model['replicas']
        if 'fullyLabeledReplicas' in model:
            self.fully_labeled_replicas = model['fullyLabeledReplicas']
        if 'readyReplicas' in model:
            self.ready_replicas = model['readyReplicas']
        if 'observedGeneration' in model:
            self.observed_generation = model['observedGeneration']

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('ReplicaSetStatus: replicas: [ {} ] is invalid.'.format(reps))
        self._replicas = reps

    # ------------------------------------------------------------------------------------- fullyLabeledReplicas

    @property
    def fully_labeled_replicas(self):
        return self._fully_labeled_replicas

    @fully_labeled_replicas.setter
    def fully_labeled_replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('ReplicaSetStatus: fully_labeled_replicas: [ {} ] is invalid.'.format(reps))
        self._fully_labeled_replicas = reps

    # ------------------------------------------------------------------------------------- readyReplicas

    @property
    def ready_replicas(self):
        return self._ready_replicas

    @ready_replicas.setter
    def ready_replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('ReplicaSetStatus: ready_replicas: [ {} ] is invalid.'.format(reps))
        self._ready_replicas = reps

    # ------------------------------------------------------------------------------------- observedGeneration

    @property
    def observed_generation(self):
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, og=None):
        if not isinstance(og, int):
            raise SyntaxError('ReplicaSetStatus: observed_generation: [ {} ] is invalid.'.format(og))
        self._observed_generation = og

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.replicas is not None:
            data['replicas'] = self.replicas
        if self.fully_labeled_replicas is not None:
            data['fullyLabeledReplicas'] = self.fully_labeled_replicas
        if self.ready_replicas is not None:
            data['readyReplicas'] = self.ready_replicas
        if self.observed_generation is not None:
            data['observedGeneration'] = self.observed_generation
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class ContainerImage(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerimage
    """

    def __init__(self, model=None):
        super(ContainerImage, self).__init__()

        self._names = None
        self._size_bytes = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'names' in model:
            self.names = model['names']
        if 'sizeBytes' in model:
            self.size_bytes = model['sizeBytes']

    # ------------------------------------------------------------------------------------- names

    @property
    def names(self):
        return self._names

    @names.setter
    def names(self, v):
        if not isinstance(v, list):
            raise SyntaxError('ContainerImage: names: [ {0} ] is invalid.'.format(v))
        self._names = v

    # ------------------------------------------------------------------------------------- size_bytes

    @property
    def size_bytes(self):
        return self._size_bytes

    @size_bytes.setter
    def size_bytes(self, v):
        if not isinstance(v, int):
            raise SyntaxError('ContainerImage: size_bytes: [ {0} ] is invalid.'.format(v))
        self._size_bytes = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.names:
            data['names'] = self.names
        if self.size_bytes:
            data['sizeBytes'] = self.size_bytes
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model, is_valid_date_time


class NodeCondition(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodecondition
    """

    def __init__(self, model=None):
        super(NodeCondition, self).__init__()

        self._condition_type = None
        self._status = None
        self._last_heartbeat_time = None
        self._last_transition_time = None
        self._reason = None
        self._message = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'type' in model:
            self.condition_type = model['type']
        if 'status' in model:
            self.status = model['status']
        if 'lastHeartbeatTime' in model:
            self.last_heartbeat_time = model['lastHeartbeatTime']
        if 'lastTransitionTime' in model:
            self.last_transition_time = model['lastTransitionTime']
        if 'reason' in model:
            self.reason = model['reason']
        if 'message' in model:
            self.message = model['message']

    # ------------------------------------------------------------------------------------- type

    @property
    def condition_type(self):
        return self._condition_type

    @condition_type.setter
    def condition_type(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeCondition: condition_type: [ {0} ] is invalid.'.format(v))
        self._condition_type = v

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeCondition: status: [ {0} ] is invalid.'.format(v))
        self._status = v

    # ------------------------------------------------------------------------------------- last_heartbeat_time

    @property
    def last_heartbeat_time(self):
        return self._last_heartbeat_time

    @last_heartbeat_time.setter
    def last_heartbeat_time(self, v):
        if not isinstance(v, str) or not is_valid_date_time(v):
            raise SyntaxError('NodeCondition: last_heartbeat_time: [ {0} ] is invalid.'.format(v))
        self._last_heartbeat_time = v

    # ------------------------------------------------------------------------------------- last_transition_time

    @property
    def last_transition_time(self):
        return self._last_transition_time

    @last_transition_time.setter
    def last_transition_time(self, v):
        if not isinstance(v, str) or not is_valid_date_time(v):
            raise SyntaxError('NodeCondition: last_transition_time: [ {0} ] is invalid.'.format(v))
        self._last_transition_time = v

    # ------------------------------------------------------------------------------------- last_transition_time

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeCondition: reason: [ {0} ] is invalid.'.format(v))
        self._reason = v

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeCondition: message: [ {0} ] is invalid.'.format(v))
        self._message = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.condition_type:
            data['type'] = self.condition_type
        if self.status:
            data['status'] = self.status
        if self.last_heartbeat_time:
            data['lastHeartbeatTime'] = self.last_heartbeat_time
        if self.last_transition_time:
            data['lastTransitionTime'] = self.last_transition_time
        if self.reason:
            data['reason'] = self.reason
        if self.message:
            data['message'] = self.message
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1beta1.DaemonSet import DaemonSet
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sVolume import K8sVolume


class K8sDaemonSet(K8sObject):

    def __init__(self, config=None, name=None):

        super(K8sDaemonSet, self).__init__(
            config=config,
            name=name,
            obj_type='DaemonSet'
        )

        self.model.spec.template.metadata.labels = self.model.metadata.labels

        sel = LabelSelector()
        sel.match_labels = {'name': name}
        self.selector = sel

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sDaemonSet, self).create()
        self.get()
        return self

    def update(self):
        super(K8sDaemonSet, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sDaemonSet, self).list(labels=labels)
        daemons = list(map(lambda x: DaemonSet(x), ls))
        if pattern is not None:
            daemons = list(filter(lambda x: pattern in x.name, daemons))
        k8s = []
        for x in daemons:
            j = K8sDaemonSet(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    # -------------------------------------------------------------------------------------  get

    def get(self):
        self.model = DaemonSet(self.get_model())
        return self

    # -------------------------------------------------------------------------------------  add

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError(
                'K8sDaemonSet.add_container() container: [ {0} ] is invalid.'.format(container))

        containers = self.model.spec.template.spec.containers
        if containers is None:
            containers = []
        filtered = list(filter(lambda x: x.name != container.name, containers))
        filtered.append(container.model)
        self.model.spec.template.spec.containers = filtered
        return self

    def add_image_pull_secrets(self, secrets=None):
        self.model.spec.template.spec.add_image_pull_secrets(secrets)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, K8sVolume):
            raise SyntaxError(
                'K8sDaemonSet.add_volume() volume: [ {0} ] is invalid.'.format(volume))

        volumes = self.model.spec.template.spec.volumes
        if volume.model not in volumes:
            volumes.append(volume.model)
        self.model.spec.template.spec.volumes = volumes
        return self

    # -------------------------------------------------------------------------------------  selector

    @property
    def selector(self):
        return self.model.spec.selector

    @selector.setter
    def selector(self, selector=None):
        self.model.spec.selector = selector



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class BaseUrls(object):
    """
    Wrapper around a map of URL endpoints for each K8sObject type.

    """

    default_api_version = 'v1'
    default_apps_version = 'v1alpha1'
    default_autoscaling_version = 'v1'
    default_batch_version = 'v1'
    default_extensions_version = 'v1beta1'
    default_cron_version = 'v2alpha1'

    def __init__(self, namespace='default', api=None, apps=None, autoscaling=None, extensions=None, batch=None, cron=None):

        if api is None:
            api = self.default_api_version
        if apps is None:
            apps = self.default_apps_version
        if autoscaling is None:
            autoscaling = self.default_autoscaling_version
        if batch is None:
            batch = self.default_batch_version
        if extensions is None:
            extensions = self.default_extensions_version
        if cron is None:
            cron = self.default_cron_version

        self.urls = dict()

        # api
        self.urls['ComponentStatus'] = '/api/{0}/componentstatuses'.format(api)
        self.urls['ConfigMap'] = '/api/{0}/namespaces/{1}/configmaps'.format(api, namespace)
        self.urls['Event'] = '/api/{0}/events'.format(api)
        self.urls['Namespace'] = '/api/{0}/namespaces'.format(api)
        self.urls['Node'] = '/api/{0}/nodes'.format(api)
        self.urls['Pod'] = '/api/{0}/namespaces/{1}/pods'.format(api, namespace)
        self.urls['PersistentVolume'] = '/api/{0}/persistentvolumes'.format(api)
        self.urls['PersistentVolumeClaim'] = '/api/{0}/namespaces/{1}/persistentvolumeclaims'.format(api, namespace)
        self.urls['ReplicationController'] = '/api/{0}/namespaces/{1}/replicationcontrollers'.format(api, namespace)
        self.urls['Service'] = '/api/{0}/namespaces/{1}/services'.format(api, namespace)
        self.urls['ServiceAccount'] = '/api/{0}/namespaces/{1}/serviceaccounts'.format(api, namespace)
        self.urls['Secret'] = '/api/{0}/namespaces/{1}/secrets'.format(api, namespace)
        self.urls['StorageClass'] = '/apis/storage.k8s.io/v1beta1/storageclasses'

        # autoscaling
        self.urls['HorizontalPodAutoscaler'] = '/apis/autoscaling/{0}/namespaces/{1}/horizontalpodautoscalers'.format(autoscaling, namespace)

        # apps
        self.urls['PetSet'] = '/apis/apps/v1alpha1/namespaces/{}/petsets'.format(namespace)
        self.urls['StatefulSet'] = '/apis/apps/v1beta1/namespaces/{}/statefulsets'.format(namespace)

        # batch
        self.urls['Job'] = '/apis/batch/{0}/namespaces/{1}/jobs'.format(batch, namespace)
        self.urls['ScheduledJob'] = '/apis/batch/{0}/namespaces/{1}/scheduledjobs'.format(cron, namespace)
        self.urls['CronJob'] = '/apis/batch/{0}/namespaces/{1}/cronjobs'.format(cron, namespace)

        # extensions
        self.urls['DaemonSet'] = '/apis/extensions/{0}/namespaces/{1}/daemonsets'.format(extensions, namespace)
        self.urls['Deployment'] = '/apis/extensions/{0}/namespaces/{1}/deployments'.format(extensions, namespace)
        self.urls['ReplicaSet'] = '/apis/extensions/{0}/namespaces/{1}/replicasets'.format(extensions, namespace)

        # other
        self.urls['Volume'] = None

    def get_base_url(self, object_type=None):
        if object_type is None or object_type not in self.urls.keys():
            types = ', '.join(self.urls.keys())
            raise SyntaxError('BaseUrls: object_type: [ {0} ] must be in: [ {1} ]'.format(object_type, types))
        return self.urls[object_type]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.LoadBalancerIngress import LoadBalancerIngress
from kubernetes.utils import is_valid_list


class LoadBalancerStatus(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_loadbalancerstatus
    """

    def __init__(self, model=None):
        super(LoadBalancerStatus, self).__init__()
        self._ingress = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'ingress' in model:
            statuses = []
            for i in model['ingress']:
                status = LoadBalancerIngress(i)
                statuses.append(status)
            self.ingress = statuses

    # ------------------------------------------------------------------------------------- ingress

    @property
    def ingress(self):
        return self._ingress

    @ingress.setter
    def ingress(self, ingress=None):
        if not is_valid_list(ingress, LoadBalancerIngress):
            raise SyntaxError('LoadBalancerStatus: ingress: [ {0} ] is invalid.'.format(ingress))
        self._ingress = ingress

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.ingress is not None:
            data['ingress'] = [x.serialize() for x in self.ingress]
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class SubresourceReference(object):
    """
    https://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_subresourcereference
    """

    def __init__(self, model=None):
        super(SubresourceReference, self).__init__()

        self._kind = None
        self._name = None
        self._api_version = None
        self._subresource = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'kind' in model:
            self.kind = model['kind']
        if 'name' in model:
            self.name = model['name']
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'subresource' in model:
            self.subresource = model['subresource']

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('SubresourceReference: kind: [ {} ] is invalid.'.format(k))
        self._kind = k
        
    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n=None):
        if not is_valid_string(n):
            raise SyntaxError('SubresourceReference: name: [ {} ] is invalid.'.format(n))
        self._name = n

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('SubresourceReference: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- subresource

    @property
    def subresource(self):
        return self._subresource

    @subresource.setter
    def subresource(self, s=None):
        if not is_valid_string(s):
            raise SyntaxError('SubresourceReference: subresource: [ {} ] is invalid.'.format(s))
        self._subresource = s

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.kind is not None:
            data['kind'] = self.kind
        if self.name is not None:
            data['name'] = self.name
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.subresource is not None:
            data['subresource'] = self.subresource
        return data


#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.models.v1.DaemonEndpoint import DaemonEndpoint
from kubernetes.utils import filter_model


class NodeDaemonEndpoints(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodedaemonendpoints
    """

    def __init__(self, model=None):
        super(NodeDaemonEndpoints, self).__init__()

        self._kubelet_endpoint = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'kubeletEndpoint' in model:
            self.kubelet_endpoint = DaemonEndpoint(model=model['kubeletEndpoint'])

    # ------------------------------------------------------------------------------------- type

    @property
    def kubelet_endpoint(self):
        return self._kubelet_endpoint

    @kubelet_endpoint.setter
    def kubelet_endpoint(self, v):
        if not isinstance(v, DaemonEndpoint):
            raise SyntaxError('NodeDaemonEndpoints: kubelet_endpoint: [ {0} ] is invalid.'.format(v))
        self._kubelet_endpoint = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.kubelet_endpoint:
            data['kubeletEndpoint'] = self.kubelet_endpoint.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.Taint import Taint
from kubernetes.utils import filter_model, is_valid_list


class NodeSpec(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodespec
    """

    def __init__(self, model=None):
        super(NodeSpec, self).__init__()

        self._pod_cidr = None
        self._external_id = None
        self._provider_id = None
        self._taints = []
        self._unschedulable = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'podCIDR' in model:
            self.pod_cidr = model['podCIDR']
        if 'externalID' in model:
            self.external_id = model['externalID']
        if 'providerID' in model:
            self.provider_id = model['providerID']
        if 'taints' in model:
            taints = []
            for t in model['taints']:
                taint = Taint(t)
                taints.append(taint)
            self.taints = taints
        if 'unschedulable' in model:
            self.unschedulable = model['unschedulable']

    # ------------------------------------------------------------------------------------- pod cidr

    @property
    def pod_cidr(self):
        return self._pod_cidr

    @pod_cidr.setter
    def pod_cidr(self, cidr):
        if not isinstance(cidr, str):
            raise SyntaxError('NodeSpec: pod_cidr: [ {0} ] is invalid.'.format(cidr))
        self._pod_cidr = cidr

    # ------------------------------------------------------------------------------------- external id

    @property
    def external_id(self):
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        if not isinstance(external_id, str):
            raise SyntaxError('NodeSpec: external_id: [ {0} ] is invalid.'.format(external_id))
        self._external_id = external_id

    # ------------------------------------------------------------------------------------- provider id

    @property
    def provider_id(self):
        return self._provider_id

    @provider_id.setter
    def provider_id(self, provider_id):
        if not isinstance(provider_id, str):
            raise SyntaxError('NodeSpec: provider_id: [ {0} ] is invalid.'.format(provider_id))
        self._provider_id = provider_id

    # ------------------------------------------------------------------------------------- taints

    @property
    def taints(self):
        return self._taints

    @taints.setter
    def taints(self, t=None):
        if not is_valid_list(t, Taint):
            raise SyntaxError('NodeSpec: taints: [ {} ] is invalid.'.format(t))
        self._taints = t

    # ------------------------------------------------------------------------------------- unschedulable

    @property
    def unschedulable(self):
        return self._unschedulable

    @unschedulable.setter
    def unschedulable(self, value):
        if not isinstance(value, bool):
            raise SyntaxError('NodeSpec: unschedulable: [ {0} ] is invalid.'.format(value))
        self._unschedulable = value

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.pod_cidr:
            data['podCIDR'] = self.pod_cidr
        if self.external_id:
            data['externalID'] = self.external_id
        if self.provider_id:
            data['providerID'] = self.provider_id
        if self.taints:
            taints = []
            for t in self.taints:
                taint = t.serialize()
                taints.append(taint)
            data['taints'] = taints
        if self.unschedulable is not None:
            data['unschedulable'] = self.unschedulable
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class EventSource(object):
    """
    https://kubernetes.io/docs/api-reference/v1.5/#eventsource-v1
    """

    def __init__(self, model=None):
        super(EventSource, self).__init__()

        self._component = None
        self._host = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'component' in model:
            self._component = model['component']
        if 'host' in model:
            self._host = model['host']

    # ------------------------------------------------------------------------------------- component

    @property
    def component(self):
        return self._component

    @component.setter
    def component(self, c=None):
        if not is_valid_string(c):
            raise SyntaxError("EventSource: component: [ {} ] is invalid.".format(c))
        self._component = c

    # ------------------------------------------------------------------------------------- host

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, h=None):
        if not is_valid_string(h):
            raise SyntaxError("EventSource: host: [ {} ] is invalid.".format(h))
        self._host = h

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.component is not None:
            data['component'] = self.component
        if self.host is not None:
            data['host'] = self.host
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sVolume import K8sVolume
from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Job import Job
from kubernetes.K8sExceptions import TimedOutException


class K8sJob(K8sObject):
    """
    See http://kubernetes.io/docs/user-guide/jobs/#parallel-jobs for the three main types of Jobs.
    Also see http://kubernetes.io/docs/user-guide/jobs/#job-patterns for common use-cases.

    - For a Non-parallel job, you can leave both .completions and .parallelism unset.
      When both are unset, both are defaulted to 1.

    - For a Fixed Completion Count job, you should set .completions to the number of completions needed.
      You can set .parallelism, or leave it unset and it will default to 1.

    - For a Work Queue Job, you must leave .completions unset, and set .parallelism to a non-negative integer.

    """

    VALID_RESTART_POLICIES = ['OnFailure', 'Never']
    SCALE_WAIT_TIMEOUT_SECONDS = 120

    def __init__(self, config=None, name=None):

        super(K8sJob, self).__init__(
            config=config,
            obj_type='Job',
            name=name
        )

        self.restart_policy = 'OnFailure'

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sJob, self).create()
        self.get()
        return self

    def update(self):
        super(K8sJob, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sJob, self).list(labels=labels)
        jobs = list(map(lambda x: Job(x), ls))
        if pattern is not None:
            jobs = list(filter(lambda x: pattern in x.name, jobs))
        k8s = []
        for x in jobs:
            j = K8sJob(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Job(self.get_model())
        return self

    # -------------------------------------------------------------------------------------  add

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError(
                'K8sJob.add_container() container: [ {0} ] is invalid.'.format(container))

        containers = self.model.spec.template.spec.containers
        if containers is None:
            containers = []
        filtered = list(filter(lambda x: x.name != container.name, containers))
        filtered.append(container.model)
        self.model.spec.template.spec.containers = filtered
        return self

    def add_image_pull_secrets(self, secrets=None):
        self.model.spec.template.spec.add_image_pull_secrets(secrets=secrets)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, K8sVolume):
            raise SyntaxError(
                'K8sJob.add_volume() volume: [ {0} ] is invalid.'.format(volume))

        volumes = self.model.spec.template.spec.volumes
        if volume.model not in volumes:
            volumes.append(volume.model)
        self.model.spec.template.spec.volumes = volumes
        return self

    # ------------------------------------------------------------------------------------- scale

    def scale(self, p=None):
        self.parallelism = p
        self.update()
        self._wait_for_desired_parallelism(p)

    # ------------------------------------------------------------------------------------- wait

    def _wait_for_desired_parallelism(self, p=None):
        start_time = time.time()
        while self.parallelism != p:
            time.sleep(0.5)
            self.get()
            self._check_timeout(start_time, p)

    def _check_timeout(self, start_time=None, p=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.SCALE_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out scaling job: [ {0} ] to parallelism: [ {1} ]".format(self.name, p))

    # ------------------------------------------------------------------------------------- parallelism

    @property
    def parallelism(self):
        return self.model.spec.parallelism

    @parallelism.setter
    def parallelism(self, p=None):
        self.model.spec.parallelism = p

    # ------------------------------------------------------------------------------------- completions

    @property
    def completions(self):
        return self.model.spec.completions

    @completions.setter
    def completions(self, c=None):
        self.model.spec.completions = c

    # ------------------------------------------------------------------------------------- activeDeadlineSeconds

    @property
    def active_deadline_seconds(self):
        return self.model.spec.active_deadline_seconds

    @active_deadline_seconds.setter
    def active_deadline_seconds(self, s=None):
        self.model.spec.active_deadline_seconds = s

    # ------------------------------------------------------------------------------------- containers

    @property
    def containers(self):
        _list = []
        for c in self.model.spec.template.spec.containers:
            k8scontainer = K8sContainer(name=c.name, image=c.image)
            k8scontainer.model = c
            _list.append(k8scontainer)
        return _list

    @containers.setter
    def containers(self, containers=None):
        self.model.spec.template.spec.containers = [x.model for x in containers]

    # -------------------------------------------------------------------------------------  volumes

    @property
    def volumes(self):
        return self.model.spec.template.spec.volumes

    @volumes.setter
    def volumes(self, v=None):
        self.model.spec.template.spec.volumes = v

    # -------------------------------------------------------------------------------------  restartPolicy

    @property
    def restart_policy(self):
        return self.model.spec.template.spec.restart_policy

    @restart_policy.setter
    def restart_policy(self, policy=None):
        if policy not in self.VALID_RESTART_POLICIES:
            raise SyntaxError('K8sJob: restart_policy: [ {} ] is invalid.'.format(policy))
        self.model.spec.template.spec.restart_policy = policy

    # -------------------------------------------------------------------------------------  dnsPolicy

    @property
    def dns_policy(self):
        return self.model.spec.template.spec.dns_policy

    @dns_policy.setter
    def dns_policy(self, policy=None):
        if policy not in self.model.spec.template.spec.VALID_DNS_POLICIES:
            raise SyntaxError('K8sJob: dns_policy: [ {} ] is invalid.'.format(policy))
        self.model.spec.template.spec.dns_policy = policy

    # -------------------------------------------------------------------------------------  start time

    @property
    def start_time(self):
        return self.model.status.start_time

    @start_time.setter
    def start_time(self, t=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  completion time

    @property
    def completion_time(self):
        return self.model.status.completion_time

    @completion_time.setter
    def completion_time(self, t=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  failed

    @property
    def failed(self):
        return self.model.status.failed

    @failed.setter
    def failed(self, t=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  succeeded

    @property
    def succeeded(self):
        return self.model.status.succeeded

    @succeeded.setter
    def succeeded(self, t=None):
        raise NotImplementedError()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class ListMeta(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_unversioned_listmeta
    """

    def __init__(self, model=None):
        super(ListMeta, self).__init__()

        self._name = None
        self._self_link = None
        self._resource_version = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'selfLink' in model:
            self.self_link = model['selfLink']
        if 'resourceVersion' in model:
            self.resource_version = model['resourceVersion']

    # ------------------------------------------------------------------------------------- strip

    def strip(self):
        self._self_link = None
        self._resource_version = None

    # ------------------------------------------------------------------------------------- selfLink

    @property
    def self_link(self):
        return self._self_link

    @self_link.setter
    def self_link(self, link=None):
        if not is_valid_string(link):
            raise SyntaxError('ListMeta: self_link: [ {0} ] is invalid.'.format(link))
        self._self_link = link

    # ------------------------------------------------------------------------------------- resourceVersion

    @property
    def resource_version(self):
        return self._resource_version

    @resource_version.setter
    def resource_version(self, v=None):
        if v is not None:
            if not isinstance(v, str):
                raise SyntaxError('ListMeta: resource_version: [ {0} ] is invalid.'.format(v))
        self._resource_version = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.self_link:
            data['selfLink'] = self.self_link
        if self.resource_version:
            data['resourceVersion'] = self.resource_version
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class NamespaceSpec(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_namespacespec
    """

    def __init__(self, model=None):
        super(NamespaceSpec, self).__init__()

        self._finalizers = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'finalizers' in model:
            self.finalizers = model['finalizers']

    # --------------------------------------------------------------------------------- finalizers

    @property
    def finalizers(self):
        return self._finalizers

    @finalizers.setter
    def finalizers(self, f=None):
        if not isinstance(f, list):
            raise SyntaxError('NamespaceSpec: finalizers: [ {} ] is invalid. Must be a list of strings.'.format(f))
        self._finalizers = f

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.finalizers is not None:
            data['finalizers'] = self.finalizers
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v2alpha1.CronJobTemplate import CronJobTemplate
from kubernetes.utils import is_valid_string


class CronJobSpec(object):

    VALID_CONCURRENCY_POLICIES = ['Allow', 'Forbid', 'Replace']

    def __init__(self, model=None):
        super(CronJobSpec, self).__init__()

        self._schedule = "@hourly"
        self._job_template = CronJobTemplate()
        self._starting_deadline_seconds = None
        self._concurrency_policy = 'Allow'
        self._suspend = False

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'schedule' in model:
            self.schedule = model['schedule']
        if 'jobTemplate' in model:
            self.job_template = CronJobTemplate(model['jobTemplate'])
        if 'startingDeadlineSeconds' in model:
            self.starting_deadline_seconds = model['startingDeadlineSeconds']
        if 'concurrencyPolicy' in model:
            self.concurrency_policy = model['concurrencyPolicy']
        if 'suspend' in model:
            self.suspend = model['suspend']

    # ------------------------------------------------------------------------------------- schedule

    @property
    def schedule(self):
        return self._schedule

    @schedule.setter
    def schedule(self, s=None):
        if not is_valid_string(s):
            raise SyntaxError('CronJobSpec: schedule: [ {} ] is invalid.'.format(s))
        self._schedule = s

    # ------------------------------------------------------------------------------------- jobTemplate

    @property
    def job_template(self):
        return self._job_template

    @job_template.setter
    def job_template(self, jt=None):
        if not isinstance(jt, CronJobTemplate):
            raise SyntaxError('CronJobSpec: job_template: [ {} ] is invalid.'.format(jt))
        self._job_template = jt

    # ------------------------------------------------------------------------------------- startingDeadlineSeconds

    @property
    def starting_deadline_seconds(self):
        return self._starting_deadline_seconds

    @starting_deadline_seconds.setter
    def starting_deadline_seconds(self, sds=None):
        if not isinstance(sds, int) or not sds >= 0:
            raise SyntaxError('CronJobSpec: starting_deadline_seconds: [ {} ] is invalid.'.format(sds))
        self._starting_deadline_seconds = sds

    # ------------------------------------------------------------------------------------- concurrencyPolicy

    @property
    def concurrency_policy(self):
        return self._concurrency_policy

    @concurrency_policy.setter
    def concurrency_policy(self, cp=None):
        if cp not in self.VALID_CONCURRENCY_POLICIES:
            raise SyntaxError('CronJobSpec: concurrency_policy: [ {} ] is invalid.'.format(cp))
        self._concurrency_policy = cp

    # ------------------------------------------------------------------------------------- suspend

    @property
    def suspend(self):
        return self._suspend

    @suspend.setter
    def suspend(self, s=False):
        if not isinstance(s, bool):
            raise SyntaxError('CronJobSpec: suspend: [ {} ] is invalid.'.format(s))
        self._suspend = s

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.schedule is not None:
            data['schedule'] = self.schedule
        if self.job_template is not None:
            data['jobTemplate'] = self.job_template.serialize()
        if self.starting_deadline_seconds is not None:
            data['startingDeadlineSeconds'] = self.starting_deadline_seconds
        if self.concurrency_policy is not None:
            data['concurrencyPolicy'] = self.concurrency_policy
        if self.suspend is not None:
            data['suspend'] = self.suspend
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import copy
import time
import uuid

from kubernetes import K8sConfig
from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sExceptions import *
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPod import K8sPod
from kubernetes.K8sVolume import K8sVolume
from kubernetes.models.v1.Probe import Probe
from kubernetes.models.v1.ReplicationController import ReplicationController
from kubernetes.utils import is_valid_string


class K8sReplicationController(K8sObject):

    SCALE_WAIT_TIMEOUT_SECONDS = 600
    DESIRED_REPLICAS_ANNOTATION = 'kubernetes.io/desired-replicas'

    def __init__(self, config=None, name=None, replicas=0):

        super(K8sReplicationController, self).__init__(
            config=config,
            obj_type='ReplicationController',
            name=name
        )

        rc_version = str(uuid.uuid4())
        labels = {'name': name, 'rc_version': rc_version}

        self.labels = labels
        self.pod_labels = labels
        self.selector = labels
        self.desired_replicas = replicas

        if config is not None and config.pull_secret is not None:
            self.add_image_pull_secrets(config.pull_secret)

    # -------------------------------------------------------------------------------------  override

    def create(self):
        # _hash = base64.b64encode(self.as_json())
        # self.add_annotation('kubernetes.io/deployment', _hash)
        super(K8sReplicationController, self).create()
        if self.desired_replicas > 0:
            self._wait_for_desired_replicas()
        return self

    def update(self):
        # _hash = base64.b64encode(self.as_json())
        # self.add_annotation('kubernetes.io/deployment', _hash)
        super(K8sReplicationController, self).update()
        if self.desired_replicas > 0:
            self._wait_for_desired_replicas()
        return self

    def list(self, pattern=None):
        ls = super(K8sReplicationController, self).list()
        rcs = list(map(lambda x: ReplicationController(x), ls))
        if pattern is not None:
            rcs = list(filter(lambda x: pattern in x.name, rcs))
        k8s = []
        for x in rcs:
            j = K8sReplicationController(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # -------------------------------------------------------------------------------------  add

    def add_pod_annotation(self, k=None, v=None):
        anns = self.model.spec.template.metadata.annotations
        if anns is None:
            anns = {}
        anns.update({k: v})
        self.model.spec.template.metadata.annotations = anns
        return self

    def add_pod_label(self, k=None, v=None):
        labels = self.model.spec.template.metadata.labels
        if labels is None:
            labels = {}
        labels.update({k: v})
        self.model.spec.template.metadata.labels = labels
        return self

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError(
                'K8sReplicationController.add_container() container: [ {0} ] is invalid.'.format(container))

        containers = self.model.spec.template.spec.containers
        if containers is None:
            containers = []
        filtered = list(filter(lambda x: x.name != container.name, containers))
        filtered.append(container.model)
        self.model.spec.template.spec.containers = filtered
        return self

    def add_image_pull_secrets(self, secrets=None):
        self.model.spec.template.spec.add_image_pull_secrets(secrets)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, K8sVolume):
            raise SyntaxError(
                'K8sReplicationController.add_volume() volume: [ {0} ] is invalid.'.format(volume))

        volumes = self.model.spec.template.spec.volumes
        if volume.model not in volumes:
            volumes.append(volume.model)
        self.model.spec.template.spec.volumes = volumes
        return self

    # -------------------------------------------------------------------------------------  del

    def del_pod_annotation(self, k=None):
        anns = self.model.spec.template.metadata.annotations
        if k in anns:
            anns.pop(k)
            self.model.spec.template.metadata.annotations = anns
        return self

    def del_pod_label(self, k=None):
        labels = self.model.spec.template.metadata.labels
        if k in labels:
            labels.pop(k)
            self.model.spec.template.metadata.labels = labels
        return self

    def del_pod_node_name(self):
        self.model.spec.template.spec.del_node_name()
        return self

    # -------------------------------------------------------------------------------------  get

    def get(self):
        self.model = ReplicationController(self.get_model())
        return self

    def get_pod_annotation(self, k=None):
        if k in self.pod_annotations:
            return self.pod_annotations[k]
        return None

    def get_pod_label(self, k=None):
        if k in self.pod_labels:
            return self.pod_labels[k]
        return None

    # -------------------------------------------------------------------------------------  activeDeadlineSeconds

    @property
    def active_deadline(self):
        return self.model.spec.template.spec.active_deadline_seconds

    @active_deadline.setter
    def active_deadline(self, secs=None):
        self.model.spec.template.spec.active_deadline_seconds = secs

    # -------------------------------------------------------------------------------------  containers

    @property
    def containers(self):
        objs = []
        for c in self.model.spec.template.spec.containers:
            k8scontainer = K8sContainer(name=c.name, image=c.image)
            k8scontainer.model = c
            objs.append(k8scontainer)
        return objs

    @containers.setter
    def containers(self, containers=None):
        models = []
        for obj in containers:
            models.append(obj.model)
        self.model.spec.template.spec.containers = models

    # -------------------------------------------------------------------------------------  container_image

    @property
    def container_image(self):
        data = {}
        for c in self.containers:
            data[c.name] = c.image
        return data

    @container_image.setter
    def container_image(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sReplicationController.container_image() must be a tuple of the form (name, image)')

        name, image = tup
        found = list(filter(lambda x: x.name == name, self.containers))
        if found:
            new = list(filter(lambda x: x.name != name, self.containers))
            found[0].image = image
            new.append(found[0])
            self.containers = new

    # -------------------------------------------------------------------------------------  image pull secrets

    @property
    def image_pull_secrets(self):
        return self.model.spec.template.spec.image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, secrets=None):
        self.model.spec.template.spec.image_pull_secrets = secrets

    # -------------------------------------------------------------------------------------  namespace

    @property
    def namespace(self):
        return self.model.metadata.namespace

    @namespace.setter
    def namespace(self, nspace=None):
        self.model.metadata.namespace = nspace

    # -------------------------------------------------------------------------------------  nodeSelector

    @property
    def node_selector(self):
        return self.model.spec.template.spec.node_selector

    @node_selector.setter
    def node_selector(self, sel=None):
        self.model.spec.template.spec.node_selector = sel

    # -------------------------------------------------------------------------------------  pod annotations

    @property
    def pod_annotations(self):
        return self.model.spec.template.metadata.annotations

    @pod_annotations.setter
    def pod_annotations(self, anns=None):
        self.model.spec.template.metadata.annotations = anns

    # -------------------------------------------------------------------------------------  pod labels

    @property
    def pod_labels(self):
        return self.model.spec.template.metadata.labels

    @pod_labels.setter
    def pod_labels(self, labels=None):
        self.model.spec.template.metadata.labels = labels

    # -------------------------------------------------------------------------------------  pod generate name

    @property
    def pod_generate_name(self):
        return self.model.spec.template.metadata.generate_name

    @pod_generate_name.setter
    def pod_generate_name(self, name=None):
        self.model.spec.template.metadata.generate_name = name

    # -------------------------------------------------------------------------------------  pod name

    @property
    def pod_name(self):
        return self.model.spec.template.metadata.name

    @pod_name.setter
    def pod_name(self, name=None):
        self.model.spec.template.metadata.name = name
        self.model.spec.template.metadata.labels['name'] = name

    # -------------------------------------------------------------------------------------  pod node name

    @property
    def pod_node_name(self):
        return self.model.spec.template.spec.node_name

    @pod_node_name.setter
    def pod_node_name(self, name=None):
        self.model.spec.template.spec.node_name = name

    # -------------------------------------------------------------------------------------  desiredReplicas

    @property
    def desired_replicas(self):
        return self.model.spec.replicas

    @desired_replicas.setter
    def desired_replicas(self, reps=None):
        self.model.spec.replicas = reps

    # -------------------------------------------------------------------------------------  dnsPolicy

    @property
    def dns_policy(self):
        return self.model.spec.template.spec.dns_policy

    @dns_policy.setter
    def dns_policy(self, policy='Default'):
        self.model.spec.template.spec.dns_policy = policy

    # -------------------------------------------------------------------------------------  observedReplicas

    @property
    def current_replicas(self):
        return self.model.status.replicas

    @current_replicas.setter
    def current_replicas(self, reps=None):
        self.model.status.replicas = reps

    # -------------------------------------------------------------------------------------  livenessProbes

    @property
    def liveness_probes(self):
        data = {}
        containers = self.model.spec.template.spec.containers
        for c in containers:
            if c.liveness_probe is not None:
                data[c.name] = c.liveness_probe
        return data

    @liveness_probes.setter
    def liveness_probes(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sReplicationController: liveness_probes: [ {} ] is invalid.'.format(tup))

        c_name, probe = tup
        container_names = [c.name for c in self.model.spec.template.spec.containers]

        if c_name not in container_names:
            raise SyntaxError(
                'K8sReplicationController: liveness_probes: container [ {} ] not found.'.format(c_name))
        if not isinstance(probe, Probe):
            raise SyntaxError(
                'K8sReplicationController: liveness_probe: probe: [ {} ] is invalid.'.format(probe))

        containers = []
        for c in self.model.spec.template.spec.containers:
            if c.name == c_name:
                c.liveness_probe = probe
            containers.append(c)
        self.model.spec.template.spec.containers = containers

    # -------------------------------------------------------------------------------------  readinessProbes

    @property
    def readiness_probes(self):
        data = {}
        containers = self.model.spec.template.spec.containers
        for c in containers:
            if c.readiness_probe is not None:
                data[c.name] = c.readiness_probe
        return data

    @readiness_probes.setter
    def readiness_probes(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sReplicationController: readiness_probes: [ {} ] is invalid.'.format(tup))

        c_name, probe = tup
        container_names = [c.name for c in self.model.spec.template.spec.containers]

        if c_name not in container_names:
            raise SyntaxError(
                'K8sReplicationController: readiness_probes: container [ {} ] not found.'.format(c_name))
        if not isinstance(probe, Probe):
            raise SyntaxError(
                'K8sReplicationController: readiness_probes: probe: [ {} ] is invalid.'.format(probe))

        containers = []
        for c in self.model.spec.template.spec.containers:
            if c.name == c_name:
                c.readiness_probe = probe
            containers.append(c)
        self.model.spec.template.spec.containers = containers

    # -------------------------------------------------------------------------------------  readyReplicas

    @property
    def ready_replicas(self):
        return self.model.status.ready_replicas

    @ready_replicas.setter
    def ready_replicas(self, reps=None):
        self.model.status.ready_replicas = reps

    # -------------------------------------------------------------------------------------  restartPolicy

    @property
    def restart_policy(self):
        return self.model.spec.template.spec.restart_policy

    @restart_policy.setter
    def restart_policy(self, policy=None):
        self.model.spec.template.spec.restart_policy = policy

    # -------------------------------------------------------------------------------------  selector

    @property
    def selector(self):
        return self.model.spec.selector

    @selector.setter
    def selector(self, selector=None):
        self.model.spec.selector = selector

    # -------------------------------------------------------------------------------------  serviceAccountName

    @property
    def service_account_name(self):
        return self.model.spec.template.spec.service_account_name

    @service_account_name.setter
    def service_account_name(self, acct=None):
        self.model.spec.template.spec.service_account_name = acct

    # -------------------------------------------------------------------------------------  terminationGracePeriod

    @property
    def termination_grace_period(self):
        return self.model.spec.template.spec.termination_grace_period_seconds

    @termination_grace_period.setter
    def termination_grace_period(self, secs=None):
        self.model.spec.template.spec.termination_grace_period_seconds = secs

    # -------------------------------------------------------------------------------------  volumes

    @property
    def volumes(self):
        return self.model.spec.template.spec.volumes

    @volumes.setter
    def volumes(self, v=None):
        self.model.spec.template.spec.volumes = v

    # -------------------------------------------------------------------------------------  wait

    def _wait_for_desired_replicas(self):
        start_time = time.time()
        is_ready = False
        while not is_ready:
            time.sleep(0.5)
            self.get()
            is_ready = self._check_pod_readiness()
            self._check_timeout(start_time)
        return self

    def _check_pod_readiness(self):
        if self.current_replicas == self.desired_replicas:
            pods = K8sPod.get_by_labels(config=self.config, labels=self.pod_labels)
            for pod in pods:
                try:
                    if not pod.is_ready():
                        raise PodNotReadyException(pod.name)
                except NotFoundException:
                    pass
                except PodNotReadyException:
                    return False
            return True
        return False

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.SCALE_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out scaling RC: [ {0} ] "
                "to replica count: [ {1} ]".format(self.name, self.desired_replicas))

    # -------------------------------------------------------------------------------------  get pods

    def get_pods(self):
        pods = K8sPod.get_by_labels(config=self.config, labels=self.pod_labels)
        return pods

    # -------------------------------------------------------------------------------------  get by name

    @staticmethod
    def get_by_name(config=None, name=None, name_label='name'):
        if config is not None and not isinstance(config, K8sConfig):
            raise SyntaxError(
                'ReplicationController.get_by_name(): config: [ {0} ] is invalid.'.format(config))
        if not is_valid_string(name):
            raise SyntaxError(
                'K8sReplicationController.get_by_name() name: [ {0} ] is invalid.'.format(name))

        rc_list = []
        data = {'labelSelector': '{0}={1}'.format(name_label, name)}
        rcs = K8sReplicationController(config=config, name=name).get_with_params(data=data)

        for rc in rcs:
            try:
                model = ReplicationController(rc)
                obj = K8sReplicationController(config=config, name=model.metadata.name).from_model(m=model)
                rc_list.append(obj)
            except NotFoundException:
                pass

        return rc_list

    # -------------------------------------------------------------------------------------  scale

    @staticmethod
    def scale(config=None, name=None, replicas=None):
        """
        Scales the number of pods in the specified K8sReplicationController to the desired replica count.

        :param config: an instance of K8sConfig

        :param name: the name of the ReplicationController we want to scale.

        :param replicas: the desired number of replicas.

        :return: An instance of K8sReplicationController
        """

        rc = K8sReplicationController(config=config, name=name).get()
        rc.desired_replicas = replicas
        rc.update()
        rc._wait_for_desired_replicas()
        return rc

    # -------------------------------------------------------------------------------------  rolling update

    @staticmethod
    def rolling_update(config=None, name=None, image=None, container_name=None, rc_new=None):
        """
        Performs a simple rolling update of a ReplicationController.

        See https://github.com/kubernetes/kubernetes/blob/master/docs/design/simple-rolling-update.md
        for algorithm details. We have modified it slightly to allow for keeping the same RC name
        between updates, which is not supported by default by kubectl.

        :param config: An instance of K8sConfig. If omitted, reads from ~/.kube/config.

        :param name: The name of the ReplicationController we want to update.

        :param image: The updated image version we want applied.

        :param container_name: The name of the container we're targeting for the update.
               Required if more than one container is present.

        :param rc_new: An instance of K8sReplicationController with the new configuration to apply.
               Mutually exclusive with [image, container_name] if specified.

        :return:
        """

        if name is None:
            raise SyntaxError(
                'K8sReplicationController: name: [ {0} ] cannot be None.'.format(name))
        if image is None and rc_new is None:
            raise SyntaxError(
                "K8sReplicationController: please specify either 'image' or 'rc_new'")
        if container_name is not None and image is not None and rc_new is not None:
            raise SyntaxError(
                'K8sReplicationController: rc_new is mutually exclusive with an (container_name, image) pair.')

        return K8sReplicationController._rolling_update_init(
            config=config,
            name=name,
            image=image,
            container_name=container_name,
            rc_new=rc_new)

    @staticmethod
    def _rolling_update_init(config=None, name=None, image=None, container_name=None, rc_new=None):

        foo = None
        foo_next = None
        name_next = "{}-next".format(name)

        try:
            foo = K8sReplicationController(config=config, name=name).get()
        except NotFoundException:
            pass
        try:
            foo_next = K8sReplicationController(config=config, name=name_next).get()
        except NotFoundException:
            pass

        if foo is None and foo_next is None:
            raise NotFoundException(
                'K8sReplicationController.rolling_update() RC: [ {} ] not found.'.format(name))

        if foo is not None and foo_next is None:

            if rc_new is not None:
                foo_next = copy.deepcopy(rc_new)

            else:
                foo_next = K8sReplicationController(config=config, name=name)

            foo_old = copy.deepcopy(foo)
            foo_old.name = "{}-old".format(foo.name)
            foo_old.selector = copy.deepcopy(foo.selector)
            foo_old.pod_labels = copy.deepcopy(foo.pod_labels)

            foo.delete(cascade=False)
            foo_old.create()

            if image and len(foo_old.containers) > 1 and not container_name:
                raise UnprocessableEntityException(
                    'K8sReplicationController: Please specify the target container_name '
                    'on which to apply image: [ {} ].'.format(image))

            if len(foo_old.containers) == 1 and not container_name:
                container_name = foo_old.containers[0].name

            if container_name and image:
                existing = list(filter(lambda x: x.name != container_name, foo_old.containers))
                if existing:
                    [foo_next.add_container(x) for x in existing]
                filtered = list(filter(lambda x: x.name == container_name, foo_old.containers))
                if filtered:
                    container = filtered[0]
                    container.image = image
                    foo_next.add_container(container)

            new_version = str(uuid.uuid4())
            foo_next.name = name
            foo_next.add_pod_label(k='name', v=name)
            foo_next.add_pod_label(k='rc_version', v=new_version)
            foo_next.selector = {'name': name, 'rc_version': new_version}
            foo_next.add_annotation(K8sReplicationController.DESIRED_REPLICAS_ANNOTATION, foo_old.desired_replicas)
            foo_next.desired_replicas = 0

            foo_next.create()
            return K8sReplicationController._rolling_update_rollout(config, name)

        if foo is None and foo_next is not None:
            return K8sReplicationController._rolling_update_rename(config, name)

        if foo is not None and foo_next is not None:
            if K8sReplicationController.DESIRED_REPLICAS_ANNOTATION not in foo_next.annotations:
                foo_next.add_annotation(K8sReplicationController.DESIRED_REPLICAS_ANNOTATION, foo.current_replicas)
                foo_next.update()
            return K8sReplicationController._rolling_update_rollout(config, name)

    @staticmethod
    def _rolling_update_rollout(config=None, name=None):
        name_old = "{}-old".format(name)
        foo = K8sReplicationController(config=config, name=name_old).get()
        foo_next = K8sReplicationController(config=config, name=name).get()
        desired = foo_next.get_annotation(K8sReplicationController.DESIRED_REPLICAS_ANNOTATION)

        while foo_next.current_replicas < int(desired):

            K8sReplicationController.scale(
                config=config,
                name=name,
                replicas=foo_next.current_replicas + 1)

            if foo.current_replicas > 0:
                K8sReplicationController.scale(
                    config=config,
                    name=name_old,
                    replicas=foo.current_replicas - 1)

            foo.get()
            foo_next.get()

        return K8sReplicationController._rolling_update_rename(config, name)

    @staticmethod
    def _rolling_update_rename(config=None, name=None):
        name_old = "{}-old".format(name)
        foo = K8sReplicationController(config=config, name=name_old).get()
        foo_next = K8sReplicationController(config=config, name=name).get()
        foo.delete()
        return foo_next

    @staticmethod
    def _rolling_update_abort(config=None, name=None):
        foo = None
        foo_next = None
        name_next = "{}-next".format(name)

        try:
            foo = K8sReplicationController(config=config, name=name).get()
        except NotFoundException:
            pass
        try:
            foo_next = K8sReplicationController(config=config, name=name_next).get()
        except NotFoundException:
            pass

        if foo_next is None:
            raise NotFoundException(
                'K8sReplicationController.rolling_update() '
                'No pending rollout of RC: [ {} ] to abort; perform a new rollout.'.format(name))
        if foo is None:
            raise NotFoundException(
                'K8sReplicationController.rolling_update() RC: [ {} ] not found.'.format(name))

    # -------------------------------------------------------------------------------------  bad cattle

    def restart(self):
        """
        Restart will force a rolling update of the current ReplicationController to the current revision.
        This essentially spawns a fresh copy of the RC and its pods. Useful when something is misbehaving.
        """

        rc_new = copy.deepcopy(self)
        return K8sReplicationController.rolling_update(
            config=self.config,
            name=self.name,
            rc_new=rc_new)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json
import yaml

from kubernetes.utils import is_valid_string


class DeleteOptions(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_deleteoptions
    """

    def __init__(self):
        super(DeleteOptions, self).__init__()

        # TODO(froch): add support for the below.
        # self._preconditions = None

        self._kind = 'DeleteOptions'
        self._api_version = 'v1'
        self._grace_period_seconds = 0
        self._orphan_dependents = False

    # -------------------------------------------------------------------------------------  kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=0):
        if not is_valid_string(k):
            raise SyntaxError('DeleteOptions: kind: [ {0} ] is invalid.'.format(k))
        self._kind = k

    # -------------------------------------------------------------------------------------  apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=0):
        if not is_valid_string(v):
            raise SyntaxError('DeleteOptions: api_version: [ {0} ] is invalid.'.format(v))
        self._kind = v

    # -------------------------------------------------------------------------------------  grace period seconds

    @property
    def grace_period_seconds(self):
        return self._grace_period_seconds

    @grace_period_seconds.setter
    def grace_period_seconds(self, secs=0):
        if not isinstance(secs, int):
            raise SyntaxError('DeleteOptions: grace_period_seconds: [ {0} ] is invalid.'.format(secs))
        self._grace_period_seconds = secs

    # -------------------------------------------------------------------------------------  orphanDependents

    @property
    def orphan_dependents(self):
        return self._orphan_dependents

    @orphan_dependents.setter
    def orphan_dependents(self, orphan=False):
        if not isinstance(orphan, bool):
            raise SyntaxError('DeleteOptions: orphan_dependents: [ {0} ] is invalid.'.format(orphan))
        self._orphan_dependents = orphan

    # -------------------------------------------------------------------------------------  serialize

    def serialize(self):
        data = {}
        if self.kind is not None:
            data['kind'] = self.kind
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.grace_period_seconds is not None:
            data['gracePeriodSeconds'] = self.grace_period_seconds
        if self.orphan_dependents is not None:
            data['orphanDependents'] = self.orphan_dependents
        return data

    def as_json(self):
        data = self.serialize()
        j = json.dumps(data, indent=4)
        return j

    def as_yaml(self):
        data = self.serialize()
        y = yaml.dump(data, default_flow_style=False)
        return y



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ContainerPort import ContainerPort
from kubernetes.models.v1.EnvVar import EnvVar
from kubernetes.models.v1.Probe import Probe
from kubernetes.models.v1.ResourceRequirements import ResourceRequirements
from kubernetes.models.v1.SecurityContext import SecurityContext
from kubernetes.models.v1.VolumeMount import VolumeMount
from kubernetes.utils import is_valid_list, is_valid_string, filter_model


class Container(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_container
    """

    VALID_PULL_POLICIES = ['Always', 'Never', 'IfNotPresent']

    def __init__(self, model=None):
        super(Container, self).__init__()

        self._args = None
        self._command = None
        self._env = []
        self._image = None
        self._image_pull_policy = 'IfNotPresent'
        self._liveness_probe = None
        self._name = None
        self._ports = None
        self._readiness_probe = None
        self._resources = ResourceRequirements()
        self._security_context = SecurityContext()
        self._volume_mounts = []
        self._working_dir = None

        self.resources.requests = {
            'cpu': '100m',
            'memory': '32M'
        }

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def __eq__(self, other):
        # see https://github.com/kubernetes/kubernetes/blob/release-1.3/docs/design/identifiers.md
        if isinstance(other, self.__class__):
            # Uniquely name (via a name) an object across space.
            return self.name == other.name
        return NotImplemented

    def _build_with_model(self, model=None):
        if 'args' in model:
            self.args = model['args']
        if 'command' in model:
            self.command = model['command']
        if 'env' in model:
            envs = []
            for e in model['env']:
                env = EnvVar(e)
                envs.append(env)
            self.env = envs
        if 'image' in model:
            self.image = model['image']
        if 'imagePullPolicy' in model:
            self.image_pull_policy = model['imagePullPolicy']
        if 'livenessProbe' in model:
            self.liveness_probe = Probe(model['livenessProbe'])
        if 'name' in model:
            self.name = model['name']
        if 'ports' in model:
            ports = []
            for p in model['ports']:
                port = ContainerPort(p)
                ports.append(port)
            self.ports = ports
        if 'readinessProbe' in model:
            self.readiness_probe = Probe(model['readinessProbe'])
        if 'resources' in model:
            r = ResourceRequirements(model['resources'])
            self.resources = r
        if 'securityContext' in model:
            self.security_context = SecurityContext(model['securityContext'])
        if 'terminationMessagePath' in model:
            self.termination_message_path = model['terminationMessagePath']
        if 'volumeMounts' in model:
            mounts = []
            for v in model['volumeMounts']:
                mount = VolumeMount(v)
                mounts.append(mount)
            self.volume_mounts = mounts
        if 'workingDir' in model:
            self.working_dir = model['workingDir']

    # ------------------------------------------------------------------------------------- args

    @property
    def args(self):
        return self._args

    @args.setter
    def args(self, args=None):
        if not is_valid_list(args, str):
            raise SyntaxError('Container: args: [ {0} ] is invalid.'.format(args))
        self._args = args

    # ------------------------------------------------------------------------------------- command

    @property
    def command(self):
        return self._command

    @command.setter
    def command(self, command=None):
        if not is_valid_list(command, str):
            raise SyntaxError('Container: command: [ {0} ] is invalid.'.format(command))
        self._command = command

    # ------------------------------------------------------------------------------------- env

    @property
    def env(self):
        return self._env

    @env.setter
    def env(self, env=None):
        if not is_valid_list(env, EnvVar):
            raise SyntaxError("Container: env: [ {0} ] is invalid.".format(env))
        self._env = env

    # ------------------------------------------------------------------------------------- image

    @property
    def image(self):
        return self._image

    @image.setter
    def image(self, image=None):
        if not is_valid_string(image):
            raise SyntaxError('Container: image: [ {0} ] is invalid.'.format(image))
        self._image = image

    # ------------------------------------------------------------------------------------- image pull policy

    @property
    def image_pull_policy(self):
        return self._image_pull_policy

    @image_pull_policy.setter
    def image_pull_policy(self, policy=None):
        if policy not in Container.VALID_PULL_POLICIES:
            raise SyntaxError('Container: image_pull_policy: [ {0} ] is invalid.'.format(policy))
        self._image_pull_policy = policy

    # ------------------------------------------------------------------------------------- liveness probe

    @property
    def liveness_probe(self):
        return self._liveness_probe

    @liveness_probe.setter
    def liveness_probe(self, probe=None):
        if not isinstance(probe, Probe):
            raise SyntaxError('Container: liveness_probe: [ {0} ] is invalid.'.format(probe))
        self._liveness_probe = probe

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('Container: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- ports

    @property
    def ports(self):
        return self._ports

    @ports.setter
    def ports(self, ports=None):
        if not is_valid_list(ports, ContainerPort):
            raise SyntaxError('Container: ports: [ {0} ] is invalid.'.format(ports))
        self._ports = ports

    # ------------------------------------------------------------------------------------- readiness probe

    @property
    def readiness_probe(self):
        return self._readiness_probe

    @readiness_probe.setter
    def readiness_probe(self, probe=None):
        if not isinstance(probe, Probe):
            raise SyntaxError('Container: readiness_probe: [ {0} ] is invalid.'.format(probe))
        self._readiness_probe = probe

    # ------------------------------------------------------------------------------------- resource requirements

    @property
    def resources(self):
        return self._resources

    @resources.setter
    def resources(self, resources=None):
        if not isinstance(resources, ResourceRequirements):
            raise SyntaxError('Container: resources: [ {0} ] is invalid.'.format(resources))
        self._resources = resources

    # ------------------------------------------------------------------------------------- security context

    @property
    def security_context(self):
        return self._security_context

    @security_context.setter
    def security_context(self, context=None):
        if not isinstance(context, SecurityContext):
            raise SyntaxError('Container: security_context: [ {0} ] is invalid.'.format(context))
        self._security_context = context

    # ------------------------------------------------------------------------------------- volume mounts

    @property
    def volume_mounts(self):
        return self._volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, mounts=None):
        if not is_valid_list(mounts, VolumeMount):
            raise SyntaxError('Container: volume_mounts: [ {0} ] is invalid.'.format(mounts))
        self._volume_mounts = mounts

    # ------------------------------------------------------------------------------------- working dir
    @property
    def working_dir(self):
        return self._working_dir

    @working_dir.setter
    def working_dir(self, wdir=None):
        if not is_valid_string(wdir):
            raise SyntaxError('Container: working_dir: [ {0} ] is invalid.'.format(wdir))
        self._working_dir = wdir

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.args is not None:
            data['args'] = self.args
        if self.command is not None:
            data['command'] = self.command
        if self.env is not None:
            data['env'] = [x.serialize() for x in self.env]
        if self.image is not None:
            data['image'] = self.image
        if self.image_pull_policy is not None:
            data['imagePullPolicy'] = self.image_pull_policy
        if self.liveness_probe is not None:
            data['livenessProbe'] = self.liveness_probe.serialize()
        if self.name is not None:
            data['name'] = self.name
        if self.ports is not None:
            data['ports'] = [x.serialize() for x in self.ports]
        if self.resources is not None:
            data['resources'] = self.resources.serialize()
        if self.readiness_probe is not None:
            data['readinessProbe'] = self.readiness_probe.serialize()
        if self.security_context is not None:
            data['securityContext'] = self.security_context.serialize()
        if self.volume_mounts is not None:
            data['volumeMounts'] = [x.serialize() for x in self.volume_mounts]
        if self.working_dir is not None:
            data['workingDir'] = self.working_dir
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.HorizontalPodAutoscalerStatus import HorizontalPodAutoscalerStatus

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.HorizontalPodAutoscalerSpec import HorizontalPodAutoscalerSpec
from kubernetes.models.v1.ObjectMeta import ObjectMeta


class HorizontalPodAutoscaler(BaseModel):
    """
    https://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_horizontalpodautoscaler
    """

    def __init__(self, model=None):
        super(HorizontalPodAutoscaler, self).__init__()

        self.kind = 'HorizontalPodAutoscaler'
        self.api_version = 'autoscaling/v1'

        self.metadata = ObjectMeta()
        self.spec = HorizontalPodAutoscalerSpec()
        self.status = HorizontalPodAutoscalerStatus()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(HorizontalPodAutoscaler, self).build_with_model(model)

        if 'metadata' in model:
            self.metadata = ObjectMeta(model['metadata'])
        if 'spec' in model:
            self.spec = HorizontalPodAutoscalerSpec(model['spec'])
        if 'status' in model:
            self.status = HorizontalPodAutoscalerStatus(model['status'])

    # ------------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, s=None):
        if not isinstance(s, HorizontalPodAutoscalerSpec):
            raise SyntaxError('HorizontalPodAutoscaler: spec: [ {} ] is invalid.'.format(s))
        self._spec = s

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, s=None):
        if not isinstance(s, HorizontalPodAutoscalerStatus):
            raise SyntaxError('HorizontalPodAutoscaler: status: [ {} ] is invalid.'.format(s))
        self._status = s

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(HorizontalPodAutoscaler, self).serialize()

        if self.spec is not None:
            data['spec'] = self.spec.serialize()
        if self.status is not None:
            data['status'] = self.status.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import copy
import importlib
import re
import socket

from dateutil.parser import parse
from six import string_types

from kubernetes.K8sExceptions import NotFoundException
from kubernetes.utils.HttpRequest import HttpRequest

RE_VALID_IP = re.compile(
    r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$')

# if some string starts with a number, extract it in first group
RE_EXTRACT_VERSION_NUMBER = re.compile('^(\d+)')


def is_valid_string(target=None):
    if target is None:
        return False
    if not isinstance(target, string_types):
        return False
    return True


def is_valid_list(target=None, element_class=None):
    if target is None:
        return False
    if not isinstance(target, list):
        return False
    if element_class is not None:
        for x in target:
            if not isinstance(x, element_class):
                return False
    return True


def is_valid_dict(target=None, keys=None, type=None):
    if target is None:
        return False
    if not isinstance(target, dict):
        return False
    for k, v in target.items():
        if not isinstance(k, str):
            return False
        if type is not None and not isinstance(v, type):
            return False
    if keys is not None and isinstance(keys, list):
        for x in target:
            if x not in keys:
                return False
    return True


def is_valid_date_time(target=None):
    rc = True
    if target is None:
        rc = False
    else:
        try:
            parse(target)
        except ValueError:
            rc = False
            pass
    return rc


def is_valid_ip(ip=None):
    match = re.match(RE_VALID_IP, ip)
    if match is not None:
        return True
    return False


def is_reachable(cfg=None):
    try:
        trimmed = re.sub(r'https?://', '', cfg.api_host)
        sock = socket.socket()
        sock.settimeout(0.5)
        if ":" in trimmed:
            host, port = trimmed.split(':')
            address = (host, int(port)) if port is not None else (host, 80)
        else:
            host = trimmed
            address = (host, 80)
        sock.connect(address)
        return True

    except Exception as err:
        try:
            req = HttpRequest(
                host=cfg.api_host,
                method='GET',
                auth=cfg.auth,
                cert=cfg.cert,
                ca_cert=cfg.ca_cert,
                ca_cert_data=cfg.ca_cert_data,
                token=cfg.token
            )
            r = req.send()
            return r['success']

        except Exception as err:
            return False


# Because of version number like: 1.8.8+coreos.0:
# minor is returned as 8+, which kind of breaks numeric comparisons X_x
def sanitize_version_number(v_num):
    try:
        return int(v_num)
    except ValueError:
        match = RE_EXTRACT_VERSION_NUMBER.match(v_num)
        if match.lastindex is not None and match.lastindex == 1:
            return int(match.group(1))
        else:
            raise ValueError("Unable to extract number at beginning of string [ {} ]".format(v_num))


def filter_model(model=None):
    mutable = copy.deepcopy(model)
    for key in model:
        if model[key] is None:
            mutable.pop(key)
    return mutable


def str_to_class(obj_type=None):
    import_paths = [
        "kubernetes.models.unversioned.{}".format(obj_type),
        "kubernetes.models.v1.{}".format(obj_type),
        "kubernetes.models.v1alpha1.{}".format(obj_type),
        "kubernetes.models.v1beta1.{}".format(obj_type),
        "kubernetes.models.v2alpha1.{}".format(obj_type),
    ]

    module = None
    for path in import_paths:
        try:
            module = importlib.import_module(path)
            break
        except ImportError:
            pass

    if module is not None:
        _class = getattr(module, obj_type)()
        return _class

    raise NotFoundException("Could not import obj_type: [ {} ]".format(obj_type))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ObjectReference import ObjectReference
from kubernetes.models.v1.LocalObjectReference import LocalObjectReference
from kubernetes.utils import is_valid_list


class ServiceAccount(BaseModel):
    """
    http://kubernetes.io/docs/user-guide/service-accounts/
    """

    def __init__(self, model=None):
        super(ServiceAccount, self).__init__()

        self.kind = 'ServiceAccount'
        self.api_version = 'v1'

        self._secrets = []
        self._image_pull_secrets = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(ServiceAccount, self).build_with_model(model)

        if 'secrets' in model:
            secrets = []
            for x in model['secrets']:
                s = ObjectReference(x)
                secrets.append(s)
            self.secrets = secrets

        if 'imagePullSecrets' in model:
            secrets = []
            for x in model['imagePullSecrets']:
                s = LocalObjectReference(x)
                secrets.append(s)
            self.image_pull_secrets = secrets

    # ------------------------------------------------------------------------------------- secrets

    @property
    def secrets(self):
        return self._secrets

    @secrets.setter
    def secrets(self, s=None):
        if not is_valid_list(s, ObjectReference):
            raise SyntaxError('ServiceAccount: secrets: [ {} ] is invalid.'.format(s))
        self._secrets = s

    # ------------------------------------------------------------------------------------- image_pull_secrets

    @property
    def image_pull_secrets(self):
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, s=None):
        if not is_valid_list(s, LocalObjectReference):
            raise SyntaxError('ServiceAccount: image_pull_secrets: [ {} ] is invalid.'.format(s))
        self._image_pull_secrets = s

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(ServiceAccount, self).serialize()

        if self.secrets is not None:
            data['secrets'] = [x.serialize() for x in self.secrets]
        if self.image_pull_secrets is not None:
            data['imagePullSecrets'] = [x.serialize() for x in self.image_pull_secrets]
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.AWSElasticBlockStoreVolumeSource import AWSElasticBlockStoreVolumeSource
from kubernetes.models.v1.EmptyDirVolumeSource import EmptyDirVolumeSource
from kubernetes.models.v1.GCEPersistentDiskVolumeSource import GCEPersistentDiskVolumeSource
from kubernetes.models.v1.GitRepoVolumeSource import GitRepoVolumeSource
from kubernetes.models.v1.HostPathVolumeSource import HostPathVolumeSource
from kubernetes.models.v1.NFSVolumeSource import NFSVolumeSource
from kubernetes.models.v1.SecretVolumeSource import SecretVolumeSource
from kubernetes.models.v1.PersistentVolumeClaimVolumeSource import PersistentVolumeClaimVolumeSource
from kubernetes.models.v1.ConfigMapVolumeSource import ConfigMapVolumeSource
from kubernetes.utils import is_valid_string, filter_model


class Volume(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_volume
    """

    VOLUME_TYPES_TO_SOURCE_MAP = {
        'awsElasticBlockStore': AWSElasticBlockStoreVolumeSource,
        'emptyDir': EmptyDirVolumeSource,
        'gcePersistentDisk': GCEPersistentDiskVolumeSource,
        'gitRepo': GitRepoVolumeSource,
        'hostPath': HostPathVolumeSource,
        'nfs': NFSVolumeSource,
        'secret': SecretVolumeSource,
        'persistentVolumeClaim': PersistentVolumeClaimVolumeSource,
        'configMap': ConfigMapVolumeSource
    }

    def __init__(self, model=None):

        # TODO(froch): add support for the below
        # self._iscsi = None
        # self._glusterfs = None
        # self._rbd = None
        # self._flex_volume = None
        # self._cinder = None
        # self._cephfs = None
        # self._flocker = None
        # self._downward_api = None
        # self._fc = None
        # self._azure_file = None

        # self._vsphere_volume
        # self._quobyte = None
        # self._azuredisk = None

        self._awsElasticBlockStore = None
        self._emptyDir = None
        self._gcePersistentDisk = None
        self._gitRepo = None
        self._hostPath = None
        self._name = None
        self._nfs = None
        self._persistentVolumeClaim = None
        self._secret = None
        self._config_map = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def __eq__(self, other):
        # see https://github.com/kubernetes/kubernetes/blob/master/docs/design/identifiers.md
        if isinstance(other, self.__class__):
            return self.name == other.name
        return NotImplemented

    def _build_with_model(self, model=None):
        if 'awsElasticBlockStore' in model:
            self.awsElasticBlockStore = AWSElasticBlockStoreVolumeSource(model['awsElasticBlockStore'])
        if 'emptyDir' in model:
            self.emptyDir = EmptyDirVolumeSource(model['emptyDir'])
        if 'gcePersistentDisk' in model:
            self.gcePersistentDisk = GCEPersistentDiskVolumeSource(model['gcePersistentDisk'])
        if 'gitRepo' in model:
            self.gitRepo = GitRepoVolumeSource(model['gitRepo'])
        if 'hostPath' in model:
            self.hostPath = HostPathVolumeSource(model['hostPath'])
        if 'name' in model:
            self.name = model['name']
        if 'nfs' in model:
            self.nfs = NFSVolumeSource(model['nfs'])
        if 'secret' in model:
            self.secret = SecretVolumeSource(model['secret'])
        if 'persistentVolumeClaim' in model:
            self.persistentVolumeClaim = PersistentVolumeClaimVolumeSource(model['persistentVolumeClaim'])
        if 'configMap' in model:
            self.configMap = ConfigMapVolumeSource(model['configMap'])

    @staticmethod
    def vol_type_to_source(vol_type=None):
        return Volume.VOLUME_TYPES_TO_SOURCE_MAP[vol_type]()

    # ------------------------------------------------------------------------------------- aws ebs

    @property
    def awsElasticBlockStore(self):
        return self._awsElasticBlockStore

    @awsElasticBlockStore.setter
    def awsElasticBlockStore(self, ebs=None):
        if not isinstance(ebs, AWSElasticBlockStoreVolumeSource):
            raise SyntaxError('Volume: aws_elastic_block_store: [ {0} ] is invalid.'.format(ebs))
        self._awsElasticBlockStore = ebs

    # ------------------------------------------------------------------------------------- configMap

    @property
    def configMap(self):
        return self._config_map

    @configMap.setter
    def configMap(self, config_map=None):
        if not isinstance(config_map, ConfigMapVolumeSource):
            raise SyntaxError('Volume: config_map: [ {0} ] is invalid.'.format(config_map))
        self._config_map = config_map

    # ------------------------------------------------------------------------------------- emptyDir

    @property
    def emptyDir(self):
        return self._emptyDir

    @emptyDir.setter
    def emptyDir(self, edir=None):
        if not isinstance(edir, EmptyDirVolumeSource):
            raise SyntaxError('Volume: empty_dir: [ {0} ] is invalid.'.format(edir))
        self._emptyDir = edir

    # ------------------------------------------------------------------------------------- gce pd

    @property
    def gcePersistentDisk(self):
        return self._gcePersistentDisk

    @gcePersistentDisk.setter
    def gcePersistentDisk(self, pd=None):
        if not isinstance(pd, GCEPersistentDiskVolumeSource):
            raise SyntaxError('Volume: gce_persistent_disk: [ {0} ] is invalid.'.format(pd))
        self._gcePersistentDisk = pd

    # ------------------------------------------------------------------------------------- gitRepo

    @property
    def gitRepo(self):
        return self._gitRepo

    @gitRepo.setter
    def gitRepo(self, repo=None):
        if not isinstance(repo, GitRepoVolumeSource):
            raise SyntaxError('Volume: git_repo: [ {0} ] is invalid.'.format(repo))
        self._gitRepo = repo

    # ------------------------------------------------------------------------------------- hostPath

    @property
    def hostPath(self):
        return self._hostPath

    @hostPath.setter
    def hostPath(self, hp=None):
        if not isinstance(hp, HostPathVolumeSource):
            raise SyntaxError('Volume: host_path: [ {0} ] is invalid.'.format(hp))
        self._hostPath = hp

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('Volume: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- nfs

    @property
    def nfs(self):
        return self._nfs

    @nfs.setter
    def nfs(self, nfs=None):
        if not isinstance(nfs, NFSVolumeSource):
            raise SyntaxError('Volume: nfs: [ {0} ] is invalid.'.format(nfs))
        self._nfs = nfs

    # ------------------------------------------------------------------------------------- secret

    @property
    def secret(self):
        return self._secret

    @secret.setter
    def secret(self, secret=None):
        if not isinstance(secret, SecretVolumeSource):
            raise SyntaxError('Volume: secret: [ {0} ] is invalid.'.format(secret))
        self._secret = secret

    # ------------------------------------------------------------------------------------- persistentVolumeClaim

    @property
    def persistentVolumeClaim(self):
        return self._persistentVolumeClaim

    @persistentVolumeClaim.setter
    def persistentVolumeClaim(self, pvc=None):
        if not isinstance(pvc, PersistentVolumeClaimVolumeSource):
            raise SyntaxError('Volume: persistentVolumeClaim: [ {0} ] is invalid.'.format(pvc))
        self._persistentVolumeClaim = pvc

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.awsElasticBlockStore is not None:
            data['awsElasticBlockStore'] = self.awsElasticBlockStore.serialize()
        if self.emptyDir is not None:
            data['emptyDir'] = self.emptyDir.serialize()
        if self.gcePersistentDisk is not None:
            data['gcePersistentDisk'] = self.gcePersistentDisk.serialize()
        if self.gitRepo is not None:
            data['gitRepo'] = self.gitRepo.serialize()
        if self.hostPath is not None:
            data['hostPath'] = self.hostPath.serialize()
        if self.name is not None:
            data['name'] = self.name
        if self.nfs is not None:
            data['nfs'] = self.nfs.serialize()
        if self.secret is not None:
            data['secret'] = self.secret.serialize()
        if self.persistentVolumeClaim is not None:
            data['persistentVolumeClaim'] = self.persistentVolumeClaim.serialize()
        if self.configMap is not None:
            data['configMap'] = self.configMap.serialize()
        return data
